@@ -106,19 +106,19 @@
 #include <unistd.h> /* for FB1200S */
 #include <errno.h> /* for FB1200S */
 
-#include <sane/sane.h>
-#include <sane/saneopts.h>
-#include <sane/sanei_scsi.h>
+#include "../include/sane/sane.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 
 #define BACKEND_NAME canon
 
-#include <sane/sanei_backend.h>
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 #define PATH_MAX	1024
 #endif
 
-#include <sane/sanei_config.h>
+#include "../include/sane/sanei_config.h"
 #define CANON_CONFIG_FILE "canon.conf"
 
 #include <canon.h>
@@ -127,7 +127,6 @@
 #define SANE_I18N(text)	text
 #endif
 
-#define MM_PER_INCH	 25.4
 
 static SANE_Byte primaryHigh[256], primaryLow[256], secondaryHigh[256],
 		 secondaryLow[256];	/* modification for FB1200S */
@@ -137,24 +136,33 @@ static CANON_Device *first_dev = NULL;
 static CANON_Scanner *first_handle = NULL;
 
 static const SANE_String_Const mode_list[] = {
-  SANE_I18N(SANE_I18N("Lineart")), SANE_I18N("Halftone"), SANE_I18N("Gray"),
-  SANE_I18N("Color"), 0
+  SANE_VALUE_SCAN_MODE_LINEART,
+  SANE_VALUE_SCAN_MODE_HALFTONE,
+  SANE_VALUE_SCAN_MODE_GRAY,
+  SANE_VALUE_SCAN_MODE_COLOR,
+  0     
 };
 
 /* modification for FS2710 */
 static const SANE_String_Const mode_list_fs2710[] = {
-  SANE_I18N("Color"), SANE_I18N("Raw"), 0
+  SANE_VALUE_SCAN_MODE_COLOR,
+  SANE_I18N("Raw"), 0
 };
 
 /* modification for FB620S */
 static const SANE_String_Const mode_list_fb620[] = {
-  SANE_I18N("Lineart"), SANE_I18N("Gray"), SANE_I18N("Color"),
+  SANE_VALUE_SCAN_MODE_LINEART,
+  SANE_VALUE_SCAN_MODE_GRAY,
+  SANE_VALUE_SCAN_MODE_COLOR,
   SANE_I18N("Fine color"), 0
 };
 
 /* modification for FB1200S */
 static const SANE_String_Const mode_list_fb1200[] = {
-  SANE_I18N("Lineart"), SANE_I18N("Gray"), SANE_I18N("Color"), 0
+  SANE_VALUE_SCAN_MODE_LINEART,
+  SANE_VALUE_SCAN_MODE_GRAY,
+  SANE_VALUE_SCAN_MODE_COLOR,
+  0
 };
 
 static const SANE_String_Const tpu_dc_mode_list[] = {
@@ -253,6 +261,8 @@ get_tpu_stat (int fd, CANON_Device * dev)
   dev->tpu.Transparency = tbuf[4 + 4 + 5] * 256 + tbuf[5 + 4 + 5];
   dev->tpu.PosNeg = tbuf[6 + 4 + 5] & 0x01;
   dev->tpu.FilmType = tbuf[7 + 4 + 5];
+  if(dev->tpu.FilmType > 3)
+    dev->tpu.FilmType = 0;
 
   DBG (11, "TPU Status: %d\n", dev->tpu.Status);
   DBG (11, "TPU ControlMode: %d\n", dev->tpu.ControlMode);
@@ -563,7 +573,7 @@ do_gamma (CANON_Scanner * s)
     strcmp (filmtype_list[1], s->val[OPT_NEGATIVE].s)
     : s->val[OPT_HNEGATIVE].w;
 
-  if (!strcmp (s->val[OPT_MODE].s, SANE_I18N("Gray")))
+  if (!strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_GRAY))
     {
       /* If scanning in gray mode, use the first curve for the
          scanner's monochrome gamma component                    */
