@@ -485,10 +485,10 @@ optionTopLeftYCallback (SANE_Option * option, SANE_Handle handle,
 /*-----------------------------------------------------------------*/
 /*
 This option controls the bot-right-x corner of the scan
-Default to 2 inches.
+Default to 215.9mm, max.
 */
 
-static SANE_Fixed optionBotRightXValue = SANE_FIX (2 * MM_IN_INCH);
+static SANE_Fixed optionBotRightXValue = SANE_FIX (215.9);
 
 static SANE_Option_Descriptor optionBotRightXDescriptor = {
   SANE_NAME_SCAN_BR_X,
@@ -529,10 +529,10 @@ optionBotRightXCallback (SANE_Option * option, SANE_Handle handle,
 /*-----------------------------------------------------------------*/
 /*
 This option controls the bot-right-y corner of the scan
-Default to 2 inches
+Default to 296.3mm, max
 */
 
-static SANE_Fixed optionBotRightYValue = SANE_FIX (2 * MM_IN_INCH);
+static SANE_Fixed optionBotRightYValue = SANE_FIX (296.3);
 
 static SANE_Option_Descriptor optionBotRightYDescriptor = {
   SANE_NAME_SCAN_BR_Y,
@@ -721,10 +721,10 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (2, "sane_init: version_code %s 0, authorize %s 0\n",
        version_code == 0 ? "=" : "!=", authorize == 0 ? "=" : "!=");
   DBG (1, "sane_init: SANE Canon630u backend version %d.%d.%d from %s\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
+       SANE_CURRENT_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
   sanei_usb_init ();
 
@@ -930,7 +930,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
   DBG (3,
        "sane_control_option: handle=%p, opt=%d, act=%d, val=%p, info=%p\n",
-       handle, option, action, value, info);
+       handle, option, action, value, (void *)info);
 
   return dispatch_control_option (handle, option, action, value, info);
 }
@@ -1037,8 +1037,7 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 SANE_Status
 sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
-  Canon_Scanner *scanner = handle;
-  DBG (3, "sane_get_select_fd: handle = %p, fd %s 0\n", handle,
-       fd ? "!=" : "=");
+  handle = handle;                   /* silence gcc */
+  fd = fd;                           /* silence gcc */
   return SANE_STATUS_UNSUPPORTED;
 }
