@@ -40,7 +40,7 @@
 
    -----
 
-   canon_pp.c: $Revision: 1.11 $
+   canon_pp.c: $Revision: 1.15 $
 
    This file is part of the canon_pp backend, supporting Canon FBX30P 
    and NX40P scanners
@@ -50,13 +50,20 @@
 #include  <lalloca.h>		/* MUST come first for AIX! */
 #endif
 
-#define VERSION "$Revision: 1.11 $"
 #define BACKEND_NAME canon_pp
 
 #define THREE_BITS 0xE0
 #define TWO_BITS 0xC0
 #define MM_PER_IN 25.4
 
+#ifndef NOSANE
+#include "../include/sane/config.h"
+#endif
+
+#ifndef VERSION
+#define VERSION "$Revision: 1.15 $"
+#endif
+
 #include  <string.h>
 #include  <math.h>
 #include  <unistd.h>
@@ -107,7 +114,11 @@ static SANE_Bool force_nibble = SANE_FALSE;
 /* Constants */
 
 /* Colour Modes */
-static const SANE_String_Const cmodes[] = { "Greyscale", "Colour", NULL };
+static const SANE_String_Const cmodes[] = { 
+	SANE_VALUE_SCAN_MODE_GRAY, 
+	SANE_VALUE_SCAN_MODE_COLOR, 
+	NULL };
+
 /* bit depths */
 static const SANE_String_Const depths[] = { "8", "12", NULL };
 /* resolutions */
@@ -139,7 +150,8 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 	DBG_INIT();
 
 #if defined PACKAGE && defined VERSION
-	DBG(2, ">> sane_init(%p, %p): " PACKAGE " " VERSION "\n", vc, cb);
+	DBG(2, ">> sane_init(%p, %p): " PACKAGE " " VERSION "\n", 
+			(const void*)vc, (const void*)cb);
 #endif
 
 	if(vc)
@@ -217,7 +229,9 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 					 * parallel port above */
 					first_dev->weights_file = tmp_wf;
 					DBG(100, "sane_init: Successfully "
-							"parsed cal.\n");
+							"parsed (old) cal, "
+							"weight file is "
+							"'%s'.\n", tmp_wf);
 					continue;
 
 				}
@@ -225,14 +239,26 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 				/* Now find which scanner wants 
 				 * this calibration file */
 				s_tmp = first_dev;
+				DBG(100, "sane_init: Finding scanner on port "
+						"'%s'\n", tmp_port+1);
 				while (s_tmp != NULL)
 				{
 					if (!strcmp(s_tmp->params.port->name,
-								tmp_port))
+								tmp_port+1))
 					{
+						DBG(100, "sane_init: Found!\n");
+						/* Now terminate the weight 
+						 * file string */
+						*tmp_port = '\0';
 						s_tmp->weights_file = tmp_wf;
 						DBG(100, "sane_init: Parsed "
-								"cal.\n");
+								"cal, for port"
+								" '%s', weight"
+								" file is '%s'"
+								".\n", 
+								s_tmp->params.
+								  port->name,
+								tmp_wf);
 						break;
 					}
 					s_tmp = s_tmp->next;
@@ -280,7 +306,7 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 
 				/* now work out which port it blongs to */
 
-				tmp_port = strstr(line+10, " ") + 1;
+				tmp_port = strstr(line+10, " ");
 
 				if (tmp_port == NULL)
 				{
@@ -441,7 +467,7 @@ sane_get_devices (const SANE_Device ***dl, SANE_Bool local)
 	CANONP_Scanner *dev;
 	int i;
 
-	DBG(2, ">> sane_get_devices (%p, %d)\n", dl, local);
+	DBG(2, ">> sane_get_devices (%p, %d)\n", (const void*)dl, local);
 
 	if (dl == NULL)
 	{
@@ -493,7 +519,7 @@ sane_open (SANE_String_Const name, SANE_Handle *h)
 	SANE_Range *tmp_range;
 	int tmp;
 
-	DBG(2, ">> sane_open (h=%p, name=\"%s\")\n", h, name);
+	DBG(2, ">> sane_open (h=%p, name=\"%s\")\n", (void *)h, name);
 
 	if ((h == NULL) || (name == NULL)) 
 	{
@@ -588,9 +614,11 @@ sane_open (SANE_String_Const name, SANE_Handle *h)
 
 	if (cs->weights_file != NULL)
 		DBG(2, "sane_open: >> load_weights(%s, %p)\n", 
-				cs->weights_file, &(cs->params));
+				cs->weights_file, 
+				(const void *)(&(cs->params)));
 	else
-		DBG(2, "sane_open: >> load_weights(NULL, %p)\n", &(cs->params));
+		DBG(2, "sane_open: >> load_weights(NULL, %p)\n", 
+				(const void *)(&(cs->params)));
 	tmp = sanei_canon_pp_load_weights(cs->weights_file, &(cs->params));
 	DBG(2, "sane_open: << %d load_weights\n", tmp);
 
@@ -691,7 +719,7 @@ sane_get_option_descriptor (SANE_Handle h, SANE_Int opt)
 	if (cs->opened == SANE_FALSE)
 	{
 		DBG(1,"sane_get_option_descriptor: That scanner (%p) ain't "
-				"open yet\n", cs);
+				"open yet\n", h);
 		return NULL;
 	}
 
@@ -716,8 +744,7 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 	int i = 0, tmp, maxresi;
 
 	DBG(2, ">> sane_control_option (h=%p, opt=%d, act=%d)\n",
-			h,opt,act);
-
+			h,opt,act); 
 	/* Do some sanity checks on the parameters 
 	 * note that val can be null for buttons */
 	if ((h == NULL) || ((val == NULL) && (opt != OPT_CAL)))   
@@ -725,7 +752,8 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 		 * frontends seem to like passing a null */
 	{
 		DBG(1,"sane_control_option: Frontend passed me a null! "
-				"(h=%p,val=%p,info=%p)\n",h,val,info);
+				"(h=%p,val=%p,info=%p)\n",(void*)h,
+				val,(void*)info);
 		return SANE_STATUS_INVAL;
 	}
 
@@ -738,14 +766,14 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 	if (cs->opened == SANE_FALSE)
 	{
 		DBG(1,"sane_control_option: That scanner (%p) ain't "
-				"open yet\n", cs);
+				"open yet\n", h);
 		return SANE_STATUS_INVAL;
 	}
 
 	if (cs->scanning == SANE_TRUE)
 	{
 		DBG(1,"sane_control_option: That scanner (%p) is scanning!\n",
-				cs);
+				h);
 		return SANE_STATUS_DEVICE_BUSY;
 	}
 
@@ -833,13 +861,11 @@ sane_control_option (SANE_Handle h, SANE_Int opt, SANE_Action act,
 					if ((cs->weights_file==NULL) ||
 							cs->cal_readonly
 					   )
-						DBG(2, ">> calibrate(%p, "
-								"NULL)\n", 
-								&(cs->params));
+						DBG(2, ">> calibrate(x, " 
+								"NULL)\n");
 					else
-						DBG(2, ">> calibrate(%p,"
+						DBG(2, ">> calibrate(x,"
 								"%s)\n",
-								&(cs->params), 
 								cs->weights_file);
 
 					if (cs->cal_readonly) tmp = 
@@ -904,14 +930,15 @@ sane_get_parameters (SANE_Handle h, SANE_Parameters *params)
 {
 	int res, max_width, max_height, max_res;
         CANONP_Scanner *cs = ((CANONP_Scanner *)h);
-	DBG(2, ">> sane_get_parameters (h=%p, params=%p)\n", h, params);
+	DBG(2, ">> sane_get_parameters (h=%p, params=%p)\n", (void*)h, 
+			(void*)params);
 
 	if (h == NULL) return SANE_STATUS_INVAL;
 
 	if (cs->opened == SANE_FALSE)
 	{
 		DBG(1,"sane_get_parameters: That scanner (%p) ain't "
-				"open yet\n", cs);
+				"open yet\n", h);
 		return SANE_STATUS_INVAL;
 	}
 
@@ -1010,7 +1037,7 @@ sane_start (SANE_Handle h)
 	if (cs->opened == SANE_FALSE)
 	{
 		DBG(1,"sane_start: That scanner (%p) ain't "
-				"open yet\n", cs);
+				"open yet\n", h);
 		return SANE_STATUS_INVAL;
 	}
 
@@ -1088,7 +1115,7 @@ sane_start (SANE_Handle h)
 
 	cs->scan.mode = cs->vals[OPT_COLOUR_MODE];
 
-	DBG(10, ">> init_scan(%p, %p)\n", &(cs->params), &(cs->scan));
+	DBG(10, ">> init_scan()\n");
 	tmp = sanei_canon_pp_init_scan(&(cs->params), &(cs->scan));
 	DBG(10, "<< %d init_scan\n", tmp);
 
@@ -1129,7 +1156,8 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	static SANE_Byte *lbuf;
 	static unsigned int bytesleft;
 
-	DBG(2, ">> sane_read (h=%p, buf=%p, maxlen=%d)\n", h, buf, maxlen);
+	DBG(2, ">> sane_read (h=%p, buf=%p, maxlen=%d)\n", h, 
+			(const void *)buf, maxlen);
 
 	/* default to returning 0 - for errors */
 	*lenp = 0;
@@ -1137,7 +1165,8 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 	if ((h == NULL) || (buf == NULL) || (lenp == NULL)) 
 	{
 		DBG(1, "sane_read: This frontend's passing me dodgy gear! "
-				"(h=%p, buf=%p, lenp=%p)\n", h, buf, lenp);
+				"(h=%p, buf=%p, lenp=%p)\n", 
+				(void*)h, (void*)buf, (void*)lenp);
 		return SANE_STATUS_INVAL;
 	}
 
@@ -1244,9 +1273,9 @@ sane_read (SANE_Handle h, SANE_Byte *buf, SANE_Int maxlen, SANE_Int *lenp)
 			cs->scan.xresolution, cs->scan.yresolution,
 			cs->scan.mode, lines);
 
-	DBG(2, ">> read_segment(%p, %p, %p, %d, %d, %d)\n",
-			&is, &(cs->params), &(cs->scan), lines,
-			cs->cal_valid, cs->scan.height - cs->lines_scanned);
+	DBG(2, ">> read_segment(x, x, x, %d, %d, %d)\n",
+			lines, cs->cal_valid, 
+			cs->scan.height - cs->lines_scanned);
 	tmp = sanei_canon_pp_read_segment(&is, &(cs->params), &(cs->scan), 
 			lines, cs->cal_valid, 
 			cs->scan.height - cs->lines_scanned);
@@ -1394,7 +1423,7 @@ sane_close (SANE_Handle h)
 	if (cs->opened == SANE_FALSE)
 	{
 		DBG(1,"sane_close: That scanner (%p) ain't "
-				"open yet\n", cs);
+				"open yet\n", h);
 		return;
 	}
 
@@ -1710,7 +1739,8 @@ sane_set_io_mode (SANE_Handle h, SANE_Bool non_blocking)
 	SANE_Status
 sane_get_select_fd (SANE_Handle h, SANE_Int *fdp)
 {
-	DBG(2, ">> sane_get_select_fd (%p, %p) (not supported)\n", h, fdp);
+	DBG(2, ">> sane_get_select_fd (%p, %p) (not supported)\n", h, 
+			(const void *)fdp);
 	DBG(2, "<< sane_get_select_fd\n");
 	return SANE_STATUS_UNSUPPORTED;
 }
