@@ -17,13 +17,10 @@
 
 #define DEBUG_DECLARE_ONLY
 
-#include "../include/sane/config.h"
-
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include "sane/config.h"
 
 #include <byteorder.h>
+#include <math.h>
 
 #include "epson2.h"
 #include "epson2-io.h"
@@ -129,17 +126,158 @@ esci_set_scan_area(Epson_Scanner * s, int x, int y, int width, int height)
 	return e2_cmd_simple(s, params, 8);
 }
 
+static int
+get_roundup_index(double frac[], int n)
+{
+	int i, index = -1;
+	double max_val = 0.0;
+
+	for (i = 0; i < n; i++) {
+
+		if (frac[i] < 0)
+			continue;
+
+		if (max_val < frac[i]) {
+			index = i;
+			max_val = frac[i];
+		}
+	}
+
+	return index;
+}
+
+static int
+get_rounddown_index(double frac[], int n)
+{
+	int i, index = -1;
+	double min_val = 1.0;
+
+	for (i = 0; i < n; i++) {
+
+		if (frac[i] > 0)
+			continue;
+
+		if (min_val > frac[i]) {
+			index = i;
+			min_val = frac[i];
+		}
+	}
+
+	return index;
+}
+
+static unsigned char
+int2cpt(int val)
+{
+	if (val >= 0) {
+
+		if (val > 127)
+			val = 127;
+
+		return (unsigned char) val;
+
+	} else {
+
+		val = -val;
+
+		if (val > 127)
+			val = 127;
+
+		return (unsigned char) (0x80 | val);
+	}
+}
+
+static void
+round_cct(double org_cct[], int rnd_cct[])
+{
+	int loop = 0;
+	int i, j, sum[3];
+	double mult_cct[9], frac[9];
+
+	for (i = 0; i < 9; i++) {
+  		mult_cct[i] = org_cct[i] * 32;
+		rnd_cct[i] = (int) floor(mult_cct[i] + 0.5);
+	}
+	
+	do {
+		for (i = 0; i < 3; i++) {
+
+			int k = i * 3;
+
+			if ((rnd_cct[k] == 11) &&
+				(rnd_cct[k] == rnd_cct[k + 1]) &&
+				(rnd_cct[k] == rnd_cct[k + 2])) {
+
+				rnd_cct[k + i]--;
+				mult_cct[k + i] = rnd_cct[k + i];
+			}
+		}
+
+		for (i = 0; i < 3; i++) {
+
+			int k = i * 3;
+
+			for (sum[i] = j = 0; j < 3; j++)
+				sum[i] += rnd_cct[k + j];
+		}
+
+		for (i = 0; i < 9; i++)
+			frac[i] = mult_cct[i] - rnd_cct[i];
+
+		for (i = 0; i < 3; i++) {
+
+			int k = i * 3;
+
+			if (sum[i] < 32) {
+
+				int index = get_roundup_index(&frac[k], 3);
+				if (index != -1) {
+					rnd_cct[k + index]++;
+					mult_cct[k + index] = rnd_cct[k + index];
+					sum[i]++;
+				}
+
+			} else if (sum[i] > 32) {
+
+				int index = get_rounddown_index(&frac[k], 3);
+				if (index != -1) {
+					rnd_cct[k + index]--;
+					mult_cct[k + index] = rnd_cct[k + index];
+					sum[i]--;
+				}
+			}
+		}
+	}
+
+	while ((++loop < 2)
+		&& ((sum[0] != 32) || (sum[1] != 32) || (sum[2] != 32)));
+}
+
+static void
+profile_to_colorcoeff(double *profile, unsigned char *color_coeff)
+{
+	int cc_idx[] = { 4, 1, 7, 3, 0, 6, 5, 2, 8 };
+	int i, color_table[9];
+
+  	round_cct(profile, color_table);
+
+  	for (i = 0; i < 9; i++)
+  		color_coeff[i] = int2cpt(color_table[cc_idx[i]]);
+}
+                                            
+                                            
 /*
  * Sends the "set color correction coefficients" command with the
  * currently selected parameters to the scanner.
  */
 
 SANE_Status
-esci_set_color_correction_coefficients(Epson_Scanner * s)
+esci_set_color_correction_coefficients(Epson_Scanner * s, SANE_Word *table)
 {
 	SANE_Status status;
 	unsigned char params[2];
-	signed char cct[9];
+	unsigned char data[9];
+	double cct[9];
 
 	DBG(8, "%s\n", __func__);
 	if (!s->hw->cmd->set_color_correction_coefficients) {
@@ -154,21 +292,23 @@ esci_set_color_correction_coefficients(Epson_Scanner * s)
 	if (status != SANE_STATUS_GOOD)
 		return status;
 
-	cct[0] = s->val[OPT_CCT_1].w;
-	cct[1] = s->val[OPT_CCT_2].w;
-	cct[2] = s->val[OPT_CCT_3].w;
-	cct[3] = s->val[OPT_CCT_4].w;
-	cct[4] = s->val[OPT_CCT_5].w;
-	cct[5] = s->val[OPT_CCT_6].w;
-	cct[6] = s->val[OPT_CCT_7].w;
-	cct[7] = s->val[OPT_CCT_8].w;
-	cct[8] = s->val[OPT_CCT_9].w;
+	cct[0] = SANE_UNFIX(table[0]);
+	cct[1] = SANE_UNFIX(table[1]);
+	cct[2] = SANE_UNFIX(table[2]);
+	cct[3] = SANE_UNFIX(table[3]);
+	cct[4] = SANE_UNFIX(table[4]);
+	cct[5] = SANE_UNFIX(table[5]);
+	cct[6] = SANE_UNFIX(table[6]);
+	cct[7] = SANE_UNFIX(table[7]);
+	cct[8] = SANE_UNFIX(table[8]);
+
+	profile_to_colorcoeff(cct, data);
 
 	DBG(11, "%s: %d,%d,%d %d,%d,%d %d,%d,%d\n", __func__,
-	    cct[0], cct[1], cct[2], cct[3],
-	    cct[4], cct[5], cct[6], cct[7], cct[8]);
+	    data[0] , data[1], data[2], data[3],
+	    data[4], data[5], data[6], data[7], data[8]);
 
-	return e2_cmd_simple(s, params, 9);
+	return e2_cmd_simple(s, data, 9);
 }
 
 SANE_Status
@@ -212,25 +352,11 @@ esci_set_gamma_table(Epson_Scanner * s)
 		}
 	}
 
-	/*
-	 * When handling inverted images, we must also invert the user
-	 * supplied gamma function. This is *not* just 255-gamma -
-	 * this gives a negative image.
-	 */
-
 	for (table = 0; table < 3; table++) {
 		gamma[0] = gamma_cmds[table];
 
-		if (s->invert_image) {
-			for (n = 0; n < 256; ++n) {
-				gamma[n + 1] =
-					255 - s->gamma_table[table][255 - n];
-			}
-		} else {
-			for (n = 0; n < 256; ++n) {
-				gamma[n + 1] = s->gamma_table[table][n];
-			}
-		}
+		for (n = 0; n < 256; ++n)
+			gamma[n + 1] = s->gamma_table[table][n];
 
 		status = e2_cmd_simple(s, params, 2);
 		if (status != SANE_STATUS_GOOD)
@@ -335,20 +461,21 @@ esci_request_extended_identity(SANE_Handle handle, unsigned char *buf)
 		return status;
 
 	DBG(1, " command level   : %c%c\n", buf[0], buf[1]);
-	DBG(1, " basic resolution: %lu\n", (u_long) le32atoh(&buf[4]));
-	DBG(1, " min resolution  : %lu\n", (u_long) le32atoh(&buf[8]));
-	DBG(1, " max resolution  : %lu\n", (u_long) le32atoh(&buf[12]));
-	DBG(1, " max pixel num   : %lu\n", (u_long) le32atoh(&buf[16]));
+	DBG(1, " basic resolution: %lu\n", (unsigned long) le32atoh(&buf[4]));
+	DBG(1, " min resolution  : %lu\n", (unsigned long) le32atoh(&buf[8]));
+	DBG(1, " max resolution  : %lu\n", (unsigned long) le32atoh(&buf[12]));
+	DBG(1, " max pixel num   : %lu\n", (unsigned long) le32atoh(&buf[16]));
 	DBG(1, " scan area       : %lux%lu\n",
-	    (u_long) le32atoh(&buf[20]), (u_long) le32atoh(&buf[24]));
+	    (unsigned long) le32atoh(&buf[20]), (unsigned long) le32atoh(&buf[24]));
 
 	DBG(1, " adf area        : %lux%lu\n",
-	    (u_long) le32atoh(&buf[28]), (u_long) le32atoh(&buf[32]));
+	    (unsigned long) le32atoh(&buf[28]), (unsigned long) le32atoh(&buf[32]));
 
 	DBG(1, " tpu area        : %lux%lu\n",
-	    (u_long) le32atoh(&buf[36]), (u_long) le32atoh(&buf[40]));
+	    (unsigned long) le32atoh(&buf[36]), (unsigned long) le32atoh(&buf[40]));
 
-	DBG(1, " main status     : 0x%02x\n", buf[44]);
+	DBG(1, " capabilities (1): 0x%02x\n", buf[44]);
+	DBG(1, " capabilities (2): 0x%02x\n", buf[45]);
 	DBG(1, " input depth     : %d\n", buf[66]);
 	DBG(1, " max output depth: %d\n", buf[67]);
 	DBG(1, " rom version     : %c%c%c%c\n",
@@ -366,32 +493,46 @@ esci_request_extended_identity(SANE_Handle handle, unsigned char *buf)
 	if (le32atoh(&buf[36]) > 0)
 		DBG(1, " TPU detected\n");
 
-	DBG(1, "status:\n");
+	if (buf[44])
+		DBG(1, "capabilities (1):\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_DLF)
+	if (buf[44] & EXT_IDTY_CAP1_DLF)
 		DBG(1, " main lamp change is supported\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_NOTFBF)
+	if (buf[44] & EXT_IDTY_CAP1_NOTFBF)
 		DBG(1, " the device is NOT flatbed\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_ADFT)
+	if (buf[44] & EXT_IDTY_CAP1_ADFT)
 		DBG(1, " page type ADF is installed\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_ADFS)
+	if (buf[44] & EXT_IDTY_CAP1_ADFS)
 		DBG(1, " ADF is duplex capable\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_ADFO)
+	if (buf[44] & EXT_IDTY_CAP1_ADFO)
 		DBG(1, " page type ADF loads from the first sheet\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_LID)
+	if (buf[44] & EXT_IDTY_CAP1_LID)
 		DBG(1, " lid type option is installed\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_TPIR)
+	if (buf[44] & EXT_IDTY_CAP1_TPIR)
 		DBG(1, " infrared scanning is supported\n");
 
-	if (buf[44] & EXT_IDTY_STATUS_PB)
+	if (buf[44] & EXT_IDTY_CAP1_PB)
 		DBG(1, " push button is supported\n");
 
+
+	if (buf[45])
+		DBG(1, "capabilities (2):\n");
+
+	if (buf[45] & EXT_IDTY_CAP2_AFF)
+		DBG(1, " ADF has auto form feed\n");
+
+	if (buf[45] & EXT_IDTY_CAP2_DFD)
+		DBG(1, " ADF has double feed detection\n");
+
+	if (buf[45] & EXT_IDTY_CAP2_ADFAS)
+		DBG(1, " ADF has auto scan\n");
+
 	return SANE_STATUS_GOOD;
 }
 
@@ -429,6 +570,9 @@ esci_request_scanner_status(SANE_Handle handle, unsigned char *buf)
 	if (buf[0] & FSF_STATUS_MAIN_WU)
 		DBG(1, " scanner is warming up\n");
 
+	if (buf[0] & FSF_STATUS_MAIN_CWU)
+		DBG(1, " warmup can be cancelled\n");
+
 
 	DBG(1, "adf status      : 0x%02x\n", buf[1]);
 
@@ -507,12 +651,12 @@ esci_set_scanning_parameter(SANE_Handle handle, unsigned char *buf)
 	params[0] = FS;
 	params[1] = 'W';
 
-	DBG(10, "resolution of main scan     : %lu\n", (u_long) le32atoh(&buf[0]));
-	DBG(10, "resolution of sub scan      : %lu\n", (u_long) le32atoh(&buf[4]));
-	DBG(10, "offset length of main scan  : %lu\n", (u_long) le32atoh(&buf[8]));
-	DBG(10, "offset length of sub scan   : %lu\n", (u_long) le32atoh(&buf[12]));
-	DBG(10, "scanning length of main scan: %lu\n", (u_long) le32atoh(&buf[16]));
-	DBG(10, "scanning length of sub scan : %lu\n", (u_long) le32atoh(&buf[20]));
+	DBG(10, "resolution of main scan     : %lu\n", (unsigned long) le32atoh(&buf[0]));
+	DBG(10, "resolution of sub scan      : %lu\n", (unsigned long) le32atoh(&buf[4]));
+	DBG(10, "offset length of main scan  : %lu\n", (unsigned long) le32atoh(&buf[8]));
+	DBG(10, "offset length of sub scan   : %lu\n", (unsigned long) le32atoh(&buf[12]));
+	DBG(10, "scanning length of main scan: %lu\n", (unsigned long) le32atoh(&buf[16]));
+	DBG(10, "scanning length of sub scan : %lu\n", (unsigned long) le32atoh(&buf[20]));
 	DBG(10, "scanning color              : %d\n", buf[24]);
 	DBG(10, "data format                 : %d\n", buf[25]);
 	DBG(10, "option control              : %d\n", buf[26]);
@@ -540,6 +684,100 @@ esci_set_scanning_parameter(SANE_Handle handle, unsigned char *buf)
 	return SANE_STATUS_GOOD;
 }
 
+/* FS S */
+
+SANE_Status
+esci_get_scanning_parameter(SANE_Handle handle, unsigned char *buf)
+{
+	Epson_Scanner *s = (Epson_Scanner *) handle;
+	SANE_Status status;
+	unsigned char params[2];
+
+	DBG(8, "%s\n", __func__);
+
+	if (buf == NULL)
+		return SANE_STATUS_INVAL;
+
+	params[0] = FS;
+	params[1] = 'S';
+
+	status = e2_txrx(s, params, 2, buf, 64);
+	if (status != SANE_STATUS_GOOD)
+		return status;
+
+	DBG(10, "resolution of main scan     : %lu\n",
+	    (u_long) le32atoh(&buf[0]));
+	DBG(10, "resolution of sub scan      : %lu\n",
+	    (u_long) le32atoh(&buf[4]));
+	DBG(10, "offset length of main scan  : %lu\n",
+	    (u_long) le32atoh(&buf[8]));
+	DBG(10, "offset length of sub scan   : %lu\n",
+	    (u_long) le32atoh(&buf[12]));
+	DBG(10, "scanning length of main scan: %lu\n",
+	    (u_long) le32atoh(&buf[16]));
+	DBG(10, "scanning length of sub scan : %lu\n",
+	    (u_long) le32atoh(&buf[20]));
+	DBG(10, "scanning color              : %d\n", buf[24]);
+	DBG(10, "data format                 : %d\n", buf[25]);
+	DBG(10, "option control              : %d\n", buf[26]);
+	DBG(10, "scanning mode               : %d\n", buf[27]);
+	DBG(10, "block line number           : %d\n", buf[28]);
+	DBG(10, "gamma correction            : %d\n", buf[29]);
+	DBG(10, "brightness                  : %d\n", buf[30]);
+	DBG(10, "color correction            : %d\n", buf[31]);
+	DBG(10, "halftone processing         : %d\n", buf[32]);
+	DBG(10, "threshold                   : %d\n", buf[33]);
+	DBG(10, "auto area segmentation      : %d\n", buf[34]);
+	DBG(10, "sharpness control           : %d\n", buf[35]);
+	DBG(10, "mirroring                   : %d\n", buf[36]);
+	DBG(10, "film type                   : %d\n", buf[37]);
+	DBG(10, "main lamp lighting mode     : %d\n", buf[38]);
+
+	return SANE_STATUS_GOOD;
+}
+
+/* ESC # */
+
+SANE_Status
+esci_enable_infrared(SANE_Handle handle)
+{
+	Epson_Scanner *s = (Epson_Scanner *) handle;
+	SANE_Status status;
+	int i;
+	unsigned char params[2];
+	unsigned char buf[64];
+
+	unsigned char seq[32] = {
+		0xCA, 0xFB, 0x77, 0x71, 0x20, 0x16, 0xDA, 0x09,
+		0x5F, 0x57, 0x09, 0x12, 0x04, 0x83, 0x76, 0x77,
+		0x3C, 0x73, 0x9C, 0xBE, 0x7A, 0xE0, 0x52, 0xE2,
+		0x90, 0x0D, 0xFF, 0x9A, 0xEF, 0x4C, 0x2C, 0x81
+	};
+
+	DBG(8, "%s\n", __func__);
+
+	status = esci_get_scanning_parameter(handle, buf);
+	if (status != SANE_STATUS_GOOD)
+		return status;
+
+	for (i = 0; i < 32; i++) {
+		buf[i] = seq[i] ^ buf[i];
+	}
+
+	params[0] = ESC;
+	params[1] = '#';
+
+	status = e2_cmd_simple(s, params, 2);
+	if (status != SANE_STATUS_GOOD)
+		return status;
+
+	status = e2_cmd_simple(s, buf, 32);
+	if (status != SANE_STATUS_GOOD)
+		return status;
+
+	return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 esci_request_command_parameter(SANE_Handle handle, unsigned char *buf)
 {
@@ -804,7 +1042,7 @@ esci_request_extended_status(SANE_Handle handle, unsigned char **data,
 		break;
 	default:
 		DBG(1, "%s: unknown reply length (%lu)\n", __func__,
-			(u_long) buf_len);
+			(unsigned long) buf_len);
 		break;
 	}
 
