@@ -62,6 +62,7 @@ enum fujitsu_Option
   OPT_VARIANCE,
 
   OPT_ADVANCED_GROUP,
+  OPT_ALD,
   OPT_COMPRESS,
   OPT_COMPRESS_ARG,
   OPT_DF_ACTION,
@@ -78,6 +79,11 @@ enum fujitsu_Option
   OPT_DUPLEX_OFFSET,
   OPT_GREEN_OFFSET,
   OPT_BLUE_OFFSET,
+  OPT_LOW_MEM,
+  OPT_SIDE,
+  OPT_SWDESKEW,
+  OPT_SWDESPECK,
+  OPT_SWCROP,
 
   OPT_ENDORSER_GROUP,
   OPT_ENDORSER,
@@ -386,6 +392,7 @@ struct fujitsu
   SANE_Range duplex_offset_range;
   SANE_Range green_offset_range;
   SANE_Range blue_offset_range;
+  SANE_Range swdespeck_range;
 
   /*endorser group*/
   SANE_Range endorser_bits_range;
@@ -447,6 +454,7 @@ struct fujitsu
   int variance;
 
   /*advanced group*/
+  int ald;
   int compress;
   int compress_arg;
   int df_action;
@@ -464,7 +472,10 @@ struct fujitsu
   int duplex_offset;
   int green_offset;
   int blue_offset;
-  int use_temp_file;
+  int low_mem;
+  int swdeskew;
+  int swdespeck;
+  int swcrop;
 
   /*endorser group*/
   int u_endorser;
@@ -491,6 +502,9 @@ struct fujitsu
   */
   SANE_Parameters params;
 
+  /* also keep a backup copy, in case the software enhancement code overwrites*/
+  SANE_Parameters params_bk;
+
   /* --------------------------------------------------------------------- */
   /* values which are set by scanning functions to keep track of pages, etc */
   int started;
@@ -504,12 +518,30 @@ struct fujitsu
   /* how far we have read */
   int bytes_rx[2];
   int lines_rx[2]; /*only used by 3091*/
+  int eof_rx[2];
+  int ili_rx[2];
+  int eom_rx;
 
   /* how far we have written */
   int bytes_tx[2];
+  int eof_tx[2];
+
+  /*size of buffers (can be smaller than above*/
+  int buff_tot[2];
+  int buff_rx[2];
+  int buff_tx[2];
 
   unsigned char * buffers[2];
 
+  /* --------------------------------------------------------------------- */
+  /* values used by the software enhancment code (deskew, crop, etc)       */
+  SANE_Status deskew_stat;
+  int deskew_vals[2];
+  double deskew_slope;
+
+  SANE_Status crop_stat;
+  int crop_vals[4];
+
   /* --------------------------------------------------------------------- */
   /* values used by the compression functions, esp. jpeg with duplex       */
   int jpeg_stage;
@@ -522,6 +554,8 @@ struct fujitsu
   /* values which used by the command and data sending functions (scsi/usb)*/
   int fd;                      /* The scanner device file descriptor.     */
   size_t rs_info;
+  int rs_eom;
+  int rs_ili;
 
   /* --------------------------------------------------------------------- */
   /* values which are used by the get hardware status command              */
@@ -631,7 +665,6 @@ struct fujitsu
 
 /* ------------------------------------------------------------------------- */
 
-#define MM_PER_INCH    25.4
 #define MM_PER_UNIT_UNFIX SANE_UNFIX(SANE_FIX(MM_PER_INCH / 1200.0))
 #define MM_PER_UNIT_FIX SANE_FIX(SANE_UNFIX(SANE_FIX(MM_PER_INCH / 1200.0)))
 
@@ -744,7 +777,7 @@ static SANE_Status mode_select_buff (struct fujitsu *s);
 
 static SANE_Status mode_select_prepick (struct fujitsu *s);
 
-static SANE_Status mode_select_overscan (struct fujitsu *s);
+static SANE_Status mode_select_auto (struct fujitsu *s);
 
 static SANE_Status set_sleep_mode(struct fujitsu *s);
 
@@ -757,11 +790,16 @@ static SANE_Status endorser (struct fujitsu *s);
 static SANE_Status set_window (struct fujitsu *s);
 static SANE_Status get_pixelsize(struct fujitsu *s);
 
+static SANE_Status update_params (struct fujitsu *s);
+static SANE_Status backup_params (struct fujitsu *s);
+static SANE_Status restore_params (struct fujitsu *s);
 static SANE_Status start_scan (struct fujitsu *s);
 
 static SANE_Status check_for_cancel(struct fujitsu *s);
 
+#ifdef SANE_FRAME_JPEG
 static SANE_Status read_from_JPEGduplex(struct fujitsu *s);
+#endif
 static SANE_Status read_from_3091duplex(struct fujitsu *s);
 static SANE_Status read_from_scanner(struct fujitsu *s, int side);
 
@@ -774,6 +812,10 @@ static SANE_Status setup_buffers (struct fujitsu *s);
 
 static SANE_Status get_hardware_status (struct fujitsu *s, SANE_Int option);
 
+static SANE_Status buffer_deskew(struct fujitsu *s, int side);
+static SANE_Status buffer_crop(struct fujitsu *s, int side);
+static SANE_Status buffer_despeck(struct fujitsu *s, int side);
+
 static void hexdump (int level, char *comment, unsigned char *p, int l);
 
 static size_t maxStringSize (const SANE_String_Const strings[]);
