@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 - 2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2007 Henning Geinitz <sane@geinitz.org>
    
    This file is part of the SANE package.
    
@@ -1100,6 +1100,106 @@ static GT68xx_Model plustek_u16b_model = {
        issue. */
 };
 
+static GT68xx_Model plustek_ops12_model = {
+  "plustek-opticpro-s12",	/* Name */
+  "Plustek",			/* Device vendor string */
+  "OpticPro S12",		/* Device model name */
+  "ccd548.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  600,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, disable backtracking */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_ALWAYS_LINEMODE	/* Which flags are needed for this scanner? */
+    /* Seems to work */
+};
+
+static GT68xx_Model plustek_ops24_model = {
+  "plustek-opticpro-s24",	/* Name */
+  "Plustek",			/* Device vendor string */
+  "OpticPro S24",	/* Device model name */
+  "ccd569.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  1200,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, disable backtracking */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 200, 100, 50, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 200, 100, 50, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (4.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  48, 24, 0,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x18, 0x1c, 0x16, 0x12, 0x18, 0x1c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_ALWAYS_LINEMODE
+  				/* Which flags are needed for this scanner? */
+    /* Works (tested by Filip Kaluza). Based on genius Colorpage Vivid 1200 X. */
+};
+
+
 static GT68xx_Model genius_vivid4_model = {
   "genius-colorpage-vivid4",	/* Name */
   "Genius",			/* Device vendor string */
@@ -1496,6 +1596,56 @@ static GT68xx_Model plustek_opticslim_m12_model = {
   GT68XX_FLAG_NO_POWER_STATUS | GT68XX_FLAG_SHEET_FED
 };
 
+static GT68xx_Model genius_sf600_model = {
+  "genius-SF600",	/* Name */
+  "Genius",				/* Device vendor string */
+  "ColorPage SF600",			/* Device model name */
+  "cism216.fw",				/* Name of the firmware file */
+  SANE_FALSE,				/* Dynamic allocation flag */
+
+  &mustek_gt6816_sheetfed_command_set,	/* Command set used by this scanner */
+
+  600,					/* maximum optical sensor resolution */
+  1200,					/* maximum motor resolution */
+  600,					/* base x-res used to calculate geometry */
+  600,					/* base y-res used to calculate geometry */
+  1200,   				/* if ydpi is equal or higher, disable backtracking */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 200, 150, 100, 0},	/* possible x-resolutions */
+  {600, 300, 200, 150, 100, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.0),		/* Start of scan area in mm  (x) */
+  SANE_FIX (9.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (10.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (140.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,				/* RGB CCD Line-distance correction in pixel */
+  0,					/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  {0x24, 0x0a, 0x23, 0x0f, 0x23, 0x0b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),			/* Default gamma value */
+
+  SANE_TRUE,				/* Is this a CIS scanner? */
+  GT68XX_FLAG_NO_POWER_STATUS | GT68XX_FLAG_SHEET_FED | GT68XX_FLAG_UNTESTED
+};
+
+/* Untested but should work according to Ryan Reading <ryanr23@gmail.com>. Based on Plustek M12 */
+
 static GT68xx_Model plustek_opticslim1200_model = {
   "plustek-opticslim-1200",	/* Name */
   "Plustek",			/* Device vendor string */
@@ -1644,6 +1794,60 @@ static GT68xx_Model visioneer_onetouch_7300_model = {
 /* Tested by Jason Novek. Based on Plustek OpticSlim 2400. */
 
 
+static GT68xx_Model genius_colorpageslim_1200_model = {
+  "genius-colorpageslim-1200",  /* Name */
+  "Genius",                      /* Device vendor string */
+  "ColorPage Slim 1200",                /* Device model name */
+  "Cis3r5b1.fw",                /* Name of the firmware file */
+  SANE_FALSE,                   /* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,   /* Command set used by this scanner */
+
+   1200,                         /* maximum optical sensor resolution */
+   2400,                         /* maximum motor resolution */
+   1200,                          /* base x-res used to calculate geometry */
+   1200,                         /* base y-res used to calculate geometry */
+   1200,                         /* if ydpi is equal or higher, disable backtracking */
+   SANE_FALSE,                   /* Use base_ydpi for all resolutions */
+
+
+
+  {1200, 600, 300, 150, 100, 50, 0},    /* possible x-resolutions */
+  {2400,1200, 600, 300, 150, 100, 50, 0},       /* possible y-resolutions */
+  {16, 12, 8, 0},                       /* possible depths in gray mode */
+  {16, 12, 8, 0},                       /* possible depths in color mode */
+
+  SANE_FIX (0.5),               /* Start of scan area in mm  (x) */
+  SANE_FIX (8.0),               /* Start of scan area in mm (y) */
+  SANE_FIX (218.0),             /* Size of scan area in mm (x) */
+  SANE_FIX (299.0),             /* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in mm (y) */
+  SANE_FIX (9.5),               /* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (0.0),               /* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),               /* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,                      /* RGB CCD Line-distance correction in pixel */
+  0,                            /* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,              /* Order of the CCD/CIS colors */
+  {0x19, 0x1a, 0x18, 0x14, 0x18, 0x12}, /* Default offset/gain */
+  {0x548, 0x513, 0x48d},        /* Default exposure parameters */
+  SANE_FIX (1.5),               /* Default gamma value */
+
+  SANE_TRUE,                    /* Is this a CIS scanner? */
+  GT68XX_FLAG_ALWAYS_LINEMODE | GT68XX_FLAG_SE_2400
+};
+/* tested by  Aleksey Nedorezov <aleksey at nedorezov.com> */
+
+
+
+
 static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x10000, 0x10000, &unknown_model},	/* used for yet unknown scanners */
   {0x055f, 0x0218, &mustek_2400ta_model},
@@ -1665,6 +1869,8 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x07b3, 0x0401, &plustek_op1248u_model},	/* Same scanner, different id? */
   {0x07b3, 0x0402, &plustek_u16b_model},
   {0x07b3, 0x0403, &plustek_u16b_model},	/* two ids? 403 seems to be more common */
+  {0x07b3, 0x040b, &plustek_ops12_model},
+  {0x07b3, 0x040e, &plustek_ops24_model},
   {0x07b3, 0x0412, &plustek_opticslim_m12_model},
   {0x07b3, 0x0413, &plustek_opticslim1200_model},
   {0x07b3, 0x0422, &plustek_opticslim2400_model},
@@ -1675,6 +1881,8 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x0458, 0x201b, &genius_vivid4x_model},
   {0x0458, 0x201d, &genius_vivid1200x_model},
   {0x0458, 0x201f, &genius_vivid1200xe_model},
+  {0x0458, 0x2021, &genius_sf600_model},
   {0x04a7, 0x0444, &visioneer_onetouch_7300_model},
+  {0x0458, 0x201E, &genius_colorpageslim_1200_model},
   {0, 0, NULL}
 };
