@@ -153,7 +153,7 @@ enum Mustek_Usb_Option
   OPT_BR_Y,			/* bottom-right y */
 
   OPT_ENHANCEMENT_GROUP,	/* 10 */
-  OPT_BRIGHTNESS,
+  OPT_THRESHOLD,
   OPT_CUSTOM_GAMMA,
   OPT_GAMMA_VECTOR,
   OPT_GAMMA_VECTOR_R,
@@ -164,14 +164,6 @@ enum Mustek_Usb_Option
   NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;
-  SANE_Word *wa;		/* word array */
-  SANE_String s;
-}
-Option_Value;
-
 typedef struct Mustek_Usb_Device
 {
   struct Mustek_Usb_Device *next;
