@@ -11,7 +11,7 @@
  * Original code taken from sane-0.71<br>
  * Copyright (C) 1997 Hypercore Software Design, Ltd.<br>
  * Also based on the work done by Rick Bronson<br>
- * Copyright (C) 2000-2006 Gerhard Jaeger <gerhard@gjaeger.de><br>
+ * Copyright (C) 2000-2007 Gerhard Jaeger <gerhard@gjaeger.de><br>
  *
  * History:
  * - 0.30 - initial version
@@ -82,6 +82,7 @@
  *        - fixed constraint_type for OPT_BUTTON
  * - 0.51 - added fine calibration caching
  *        - removed #define _PLUSTEK_USB
+ * - 0.52 - added skipDarkStrip and OPT_LOFF4DARK to frontend options
  *.
  * <hr>
  * This file is part of the SANE package.
@@ -157,7 +158,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#define BACKEND_VERSION "0.51-12"
+#define BACKEND_VERSION "0.52-9"
 
 #define BACKEND_NAME    plustek
 #include "../include/sane/sanei_access.h"
@@ -275,6 +276,8 @@ static void show_cnf( CnfDef *cnf )
 	DBG( _DBG_SANE_INIT,"skipCalibr.  : %s\n",  _YN(cnf->adj.skipCalibration));
 	DBG( _DBG_SANE_INIT,"skipFine     : %s\n",  _YN(cnf->adj.skipFine       ));
 	DBG( _DBG_SANE_INIT,"skipFineWhite: %s\n",  _YN(cnf->adj.skipFineWhite  ));
+	DBG( _DBG_SANE_INIT,"skipDarkStrip: %s\n",  _YN(cnf->adj.skipDarkStrip   ));
+	DBG( _DBG_SANE_INIT,"incDarkTarget: %s\n",  _YN(cnf->adj.incDarkTgt      ));
 	DBG( _DBG_SANE_INIT,"invertNegs.  : %s\n",  _YN(cnf->adj.invertNegatives));
 	DBG( _DBG_SANE_INIT,"dis.Speedup  : %s\n",  _YN(cnf->adj.disableSpeedup ));
 	DBG( _DBG_SANE_INIT,"pos_x        : %d\n",  cnf->adj.pos.x               );
@@ -563,12 +566,12 @@ static SANE_Status
 do_cancel( Plustek_Scanner *scanner, SANE_Bool closepipe )
 {
 	struct SIGACTION act;
-	pid_t            res;
+	SANE_Pid         res;
 
 	DBG( _DBG_PROC,"do_cancel\n" );
 	scanner->scanning = SANE_FALSE;
 
-	if( scanner->reader_pid > 0 ) {
+	if( scanner->reader_pid != -1 ) {
 
 		DBG( _DBG_PROC, ">>>>>>>> killing reader_process <<<<<<<<\n" );
 
@@ -600,7 +603,7 @@ do_cancel( Plustek_Scanner *scanner, SANE_Bool closepipe )
 #endif
 		}
 
-		scanner->reader_pid = 0;
+		scanner->reader_pid = -1;
 		DBG( _DBG_PROC,"reader_process killed\n");
 #ifndef HAVE_SETITIMER
 		usb_StartLampTimer( scanner->hw );
@@ -693,21 +696,22 @@ static SANE_Status
 init_options( Plustek_Scanner *s )
 {
 	int i;
-	Plustek_Device *dev = s->hw;
-	AdjDef         *adj = &dev->adj;
+	Plustek_Device *dev  = s->hw;
+	AdjDef         *adj  = &dev->adj;
+	DCapsDef       *caps = &dev->usbDev.Caps;
 
 	memset(s->opt, 0, sizeof(s->opt));
 
 	for( i = 0; i < NUM_OPTIONS; ++i ) {
 		s->opt[i].size = sizeof (SANE_Word);
-		s->opt[i].cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
+		s->opt[i].cap  = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
+		s->opt[i].unit = SANE_UNIT_NONE;
 	}
 
 	s->opt[OPT_NUM_OPTS].name  = SANE_NAME_NUM_OPTIONS;
 	s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
 	s->opt[OPT_NUM_OPTS].desc  = SANE_DESC_NUM_OPTIONS;
 	s->opt[OPT_NUM_OPTS].type  = SANE_TYPE_INT;
-	s->opt[OPT_NUM_OPTS].unit  = SANE_UNIT_NONE;
 	s->opt[OPT_NUM_OPTS].cap   = SANE_CAP_SOFT_DETECT;
 	s->opt[OPT_NUM_OPTS].constraint_type = SANE_CONSTRAINT_NONE;
 	s->val[OPT_NUM_OPTS].w     = NUM_OPTIONS;
@@ -733,6 +737,7 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_BIT_DEPTH].title = SANE_TITLE_BIT_DEPTH;
 	s->opt[OPT_BIT_DEPTH].desc  = SANE_DESC_BIT_DEPTH;
 	s->opt[OPT_BIT_DEPTH].type  = SANE_TYPE_INT;
+	s->opt[OPT_BIT_DEPTH].unit  = SANE_UNIT_BIT;
 	s->opt[OPT_BIT_DEPTH].size  = sizeof (SANE_Word);
 	s->opt[OPT_BIT_DEPTH].constraint_type = SANE_CONSTRAINT_WORD_LIST;
 	if( _LM9833 == dev->usbDev.HwSetting.chip )
@@ -741,6 +746,9 @@ init_options( Plustek_Scanner *s )
 		s->opt[OPT_BIT_DEPTH].constraint.word_list = bpp_lm9832_list;
 	s->val[OPT_BIT_DEPTH].w = 8;
 	
+	if (caps->workaroundFlag & _WAF_ONLY_8BIT)
+		_DISABLE(OPT_BIT_DEPTH);
+
 	/* scan source */
 	s->opt[OPT_EXT_MODE].name  = SANE_NAME_SCAN_SOURCE;
 	s->opt[OPT_EXT_MODE].title = SANE_TITLE_SCAN_SOURCE;
@@ -777,7 +785,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_RESOLUTION].desc  = SANE_DESC_SCAN_RESOLUTION;
 	s->opt[OPT_RESOLUTION].type  = SANE_TYPE_INT;
 	s->opt[OPT_RESOLUTION].unit  = SANE_UNIT_DPI;
-
 	s->opt[OPT_RESOLUTION].constraint_type  = SANE_CONSTRAINT_RANGE;
 	s->opt[OPT_RESOLUTION].constraint.range = &s->hw->dpi_range;
 	s->val[OPT_RESOLUTION].w = s->hw->dpi_range.min;
@@ -856,7 +863,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_GAMMA_VECTOR].title = SANE_TITLE_GAMMA_VECTOR;
 	s->opt[OPT_GAMMA_VECTOR].desc  = SANE_DESC_GAMMA_VECTOR;
 	s->opt[OPT_GAMMA_VECTOR].type  = SANE_TYPE_INT;
-	s->opt[OPT_GAMMA_VECTOR].unit  = SANE_UNIT_NONE;
 	s->opt[OPT_GAMMA_VECTOR].constraint_type = SANE_CONSTRAINT_RANGE;
 	s->val[OPT_GAMMA_VECTOR].wa   = &(s->gamma_table[0][0]);
 	s->opt[OPT_GAMMA_VECTOR].constraint.range = &(s->gamma_range);
@@ -867,7 +873,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_GAMMA_VECTOR_R].title = SANE_TITLE_GAMMA_VECTOR_R;
 	s->opt[OPT_GAMMA_VECTOR_R].desc  = SANE_DESC_GAMMA_VECTOR_R;
 	s->opt[OPT_GAMMA_VECTOR_R].type  = SANE_TYPE_INT;
-	s->opt[OPT_GAMMA_VECTOR_R].unit  = SANE_UNIT_NONE;
 	s->opt[OPT_GAMMA_VECTOR_R].constraint_type = SANE_CONSTRAINT_RANGE;
 	s->val[OPT_GAMMA_VECTOR_R].wa   = &(s->gamma_table[1][0]);
 	s->opt[OPT_GAMMA_VECTOR_R].constraint.range = &(s->gamma_range);
@@ -878,7 +883,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_GAMMA_VECTOR_G].title = SANE_TITLE_GAMMA_VECTOR_G;
 	s->opt[OPT_GAMMA_VECTOR_G].desc  = SANE_DESC_GAMMA_VECTOR_G;
 	s->opt[OPT_GAMMA_VECTOR_G].type  = SANE_TYPE_INT;
-	s->opt[OPT_GAMMA_VECTOR_G].unit  = SANE_UNIT_NONE;
 	s->opt[OPT_GAMMA_VECTOR_G].constraint_type = SANE_CONSTRAINT_RANGE;
 	s->val[OPT_GAMMA_VECTOR_G].wa   = &(s->gamma_table[2][0]);
 	s->opt[OPT_GAMMA_VECTOR_G].constraint.range = &(s->gamma_range);
@@ -889,7 +893,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_GAMMA_VECTOR_B].title = SANE_TITLE_GAMMA_VECTOR_B;
 	s->opt[OPT_GAMMA_VECTOR_B].desc  = SANE_DESC_GAMMA_VECTOR_B;
 	s->opt[OPT_GAMMA_VECTOR_B].type  = SANE_TYPE_INT;
-	s->opt[OPT_GAMMA_VECTOR_B].unit  = SANE_UNIT_NONE;
 	s->opt[OPT_GAMMA_VECTOR_B].constraint_type = SANE_CONSTRAINT_RANGE;
 	s->val[OPT_GAMMA_VECTOR_B].wa   = &(s->gamma_table[3][0]);
 	s->opt[OPT_GAMMA_VECTOR_B].constraint.range = &(s->gamma_range);
@@ -918,6 +921,15 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_LAMPSWITCH].type  = SANE_TYPE_BOOL;
 	s->val[OPT_LAMPSWITCH].w     = SANE_FALSE;
 
+	s->opt[OPT_LOFF4DARK].name  = "lamp-off-during-dcal";
+	s->opt[OPT_LOFF4DARK].title = SANE_I18N("Lamp off during dark calibration");;
+	s->opt[OPT_LOFF4DARK].desc  = SANE_I18N("Always switches lamp off when doing dark calibration.");
+	s->opt[OPT_LOFF4DARK].type  = SANE_TYPE_BOOL;
+	s->val[OPT_LOFF4DARK].w     = adj->skipDarkStrip;
+
+	if (dev->usbDev.Caps.Normal.DarkShadOrgY < 0)
+		_DISABLE(OPT_LOFF4DARK);
+
 	s->opt[OPT_CACHECAL].name  = "calibration-cache";
 	s->opt[OPT_CACHECAL].title = SANE_I18N("Calibration data cache");;
 	s->opt[OPT_CACHECAL].desc  = SANE_I18N("Enables or disables calibration data cache.");
@@ -928,7 +940,6 @@ init_options( Plustek_Scanner *s )
 	s->opt[OPT_CALIBRATE].title = SANE_I18N("Calibrate");;
 	s->opt[OPT_CALIBRATE].desc  = SANE_I18N("Performs calibration");
 	s->opt[OPT_CALIBRATE].type  = SANE_TYPE_BUTTON;
-	s->opt[OPT_CALIBRATE].unit = SANE_UNIT_NONE;
 	s->opt[OPT_CALIBRATE].size = sizeof (SANE_Word);
 	s->opt[OPT_CALIBRATE].constraint_type = SANE_CONSTRAINT_NONE;
 	s->opt[OPT_CALIBRATE].constraint.range = 0;
@@ -1072,8 +1083,15 @@ init_options( Plustek_Scanner *s )
 
 	/* scanner buttons */
 	for( i = OPT_BUTTON_0; i <= OPT_BUTTON_LAST; i++ ) {
-		s->opt[i].name  = "button";
-		s->opt[i].title = SANE_I18N("Scanner button");
+
+		char name [12];
+		char title [128];
+
+		sprintf (name, "button %d", i - OPT_BUTTON_0);
+		sprintf (title, "Scanner button %d", i - OPT_BUTTON_0);
+
+		s->opt[i].name  = strdup(name);
+		s->opt[i].title = strdup(title);
 		s->opt[i].desc  = SANE_I18N("This option reflects the status "
 		                            "of the scanner buttons.");
 		s->opt[i].type = SANE_TYPE_BOOL;
@@ -1311,9 +1329,9 @@ attach( const char *dev_name, CnfDef *cnf, Plustek_Device **devp )
 	/* okay, so assign the handle and the scanner type */
 	dev->fd = handle;
 	if( usb_IsSheetFedDevice( dev ))
-		dev->sane.type = "USB sheet-fed scanner";
+		dev->sane.type = SANE_I18N("sheetfed scanner");
 	else
-		dev->sane.type = "USB flatbed scanner";
+		dev->sane.type = SANE_I18N("flatbed scanner");
 
 	result = usbDev_getCaps( dev );
 	if( result < 0 ) {
@@ -1402,6 +1420,8 @@ init_config_struct( CnfDef *cnf )
 	cnf->adj.glampoff     = -1;
 	cnf->adj.blampoff     = -1;
 
+	cnf->adj.incDarkTgt = 1;
+
 	cnf->adj.graygamma = 1.0;
 	cnf->adj.rgamma    = 1.0;
 	cnf->adj.ggamma    = 1.0;
@@ -1446,7 +1466,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 	usbGetList( &usbDevs );
 
 	if( version_code != NULL )
-		*version_code = SANE_VERSION_CODE(V_MAJOR, V_MINOR, 0);
+		*version_code = SANE_VERSION_CODE(SANE_CURRENT_MAJOR, V_MINOR, 0);
 
 	fp = sanei_config_open( PLUSTEK_CONFIG_FILE );
 
@@ -1500,6 +1520,10 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 									  _INT, &config.adj.skipFine,&ival);
 			decodeVal( str, "skipFineWhite",
 									  _INT, &config.adj.skipFineWhite,&ival);
+			decodeVal( str, "skipDarkStrip",
+									  _INT, &config.adj.skipDarkStrip,&ival);
+			decodeVal( str, "incDarkTarget",
+									  _INT, &config.adj.incDarkTgt,&ival);
 			decodeVal( str, "invertNegatives",
 									  _INT, &config.adj.invertNegatives,&ival);
 			decodeVal( str, "disableSpeedup",
@@ -1775,6 +1799,14 @@ do_calibration( void *args )
 
 	for ( ; scanmode < 5; scanmode++ ) {
 
+		if (caps->workaroundFlag & _WAF_ONLY_8BIT) {
+
+			if ((modes[scanmode] == COLOR_GRAY16) ||
+			    (modes[scanmode] == COLOR_TRUE48)) {
+				continue;
+			}
+		}
+ 
 		dev->scanning.dwFlag |= SCANFLAG_Calibration;
 
 		if (SANE_STATUS_GOOD == local_sane_start(s, modes[scanmode])) {
@@ -1809,9 +1841,10 @@ SANE_Status
 sane_control_option( SANE_Handle handle, SANE_Int option,
                      SANE_Action action, void *value, SANE_Int *info )
 {
-	Plustek_Scanner         *s = (Plustek_Scanner *)handle;
-	Plustek_Device          *dev = s->hw;
-	AdjDef                  *adj = &dev->adj;
+	Plustek_Scanner         *s    = (Plustek_Scanner *)handle;
+	Plustek_Device          *dev  = s->hw;
+	AdjDef                  *adj  = &dev->adj;
+	DCapsDef                *caps = &dev->usbDev.Caps;
 	SANE_Status              status;
 	const SANE_String_Const *optval;
 	int                      scanmode;
@@ -1862,6 +1895,7 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 			case OPT_LAMPSWITCH:
 			case OPT_CUSTOM_GAMMA:
 			case OPT_LAMPOFF_ONEND:
+			case OPT_LOFF4DARK:
 			case OPT_CACHECAL:
 			case OPT_SPEEDUP:
 			case OPT_OVR_REDGAIN:
@@ -2017,6 +2051,11 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 					dev->adj.disableSpeedup = !(s->val[option].w);
 					break;
 
+				case OPT_LOFF4DARK:
+					s->val[option].w = *(SANE_Word *)value;
+					dev->adj.skipDarkStrip = !(s->val[option].w);
+					break;
+
 				case OPT_LAMPSWITCH:
 					s->val[option].w = *(SANE_Word *)value;
 					usb_LampSwitch( dev, s->val[option].w );
@@ -2132,12 +2171,15 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 					_ENABLE(OPT_CONTRAST);
 					_ENABLE(OPT_BIT_DEPTH);
 					_ENABLE(OPT_CUSTOM_GAMMA);
-					if( scanmode == COLOR_BW ) {
+					if (scanmode == COLOR_BW) {
 						_DISABLE(OPT_CONTRAST);
 						_DISABLE(OPT_CUSTOM_GAMMA);
 						_DISABLE(OPT_BIT_DEPTH);
 					}
 
+					if (caps->workaroundFlag & _WAF_ONLY_8BIT)
+						_DISABLE(OPT_BIT_DEPTH);
+
 					_DISABLE(OPT_GAMMA_VECTOR);
 					_DISABLE(OPT_GAMMA_VECTOR_R);
 					_DISABLE(OPT_GAMMA_VECTOR_G);
@@ -2492,7 +2534,7 @@ local_sane_start(Plustek_Scanner *s, int scanmode )
 		return SANE_STATUS_IO_ERROR;
 	}
 
-	DBG( _DBG_SANE_INIT, "dwflag = 0x%lx dwBytesLine = %ld \n",
+	DBG( _DBG_SANE_INIT, "dwflag = 0x%lx dwBytesLine = %ld\n",
 	                      dev->scanning.dwFlag, dev->scanning.dwBytesLine );
 	DBG( _DBG_SANE_INIT, "Lines          = %d\n", s->params.lines);
 	DBG( _DBG_SANE_INIT, "Bytes per Line = %d\n", s->params.bytes_per_line );
@@ -2586,9 +2628,9 @@ sane_start( SANE_Handle handle )
 
 	cancelRead = SANE_FALSE;
 	
-	if( s->reader_pid < 0 ) {
+	if( s->reader_pid == -1 ) {
 		DBG( _DBG_ERROR, "ERROR: could not start reader task\n" );
-	    s->scanning = SANE_FALSE;
+		s->scanning = SANE_FALSE;
 		usbDev_close( dev );
 		return SANE_STATUS_IO_ERROR;
 	}
