@@ -2,6 +2,7 @@
    Copyright (C) 1996, 1997 Andreas Beck
    Copyright (C) 2000, 2001 Michael Herder <crapsite@gmx.net>
    Copyright (C) 2001, 2002 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2008 Stphane Voltz <stef.dev@free.fr>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -40,7 +41,7 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.  */
 
-#define BUILD 8
+#define BUILD 9
 
 #include "../include/sane/config.h"
 
@@ -50,6 +51,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
@@ -79,6 +81,10 @@ static SANE_Word status_ioerror = SANE_FALSE;
 static SANE_Word status_nomem = SANE_FALSE;
 static SANE_Word status_accessdenied = SANE_FALSE;
 static SANE_Word test_option = 0;
+#ifdef SANE_STATUS_WARMING_UP
+static SANE_Word warming_up = SANE_FALSE;
+static struct timeval start;
+#endif
 
 static SANE_Fixed bright = 0;
 static SANE_Word res = 75;
@@ -505,10 +511,10 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (2, "sane_init: version_code %s 0, authorize %s 0\n",
        version_code == 0 ? "=" : "!=", authorize == 0 ? "=" : "!=");
   DBG (1, "sane_init: SANE pnm backend version %d.%d.%d from %s\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_VERSION);
+       SANE_CURRENT_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
   return SANE_STATUS_GOOD;
 }
 
@@ -532,13 +538,34 @@ static const SANE_Device dev[] = {
    "Noname",
    "PNM file reader",
    "virtual device"},
+#ifdef SANE_STATUS_HW_LOCKED
+  {
+   "locked",
+   "Noname",
+   "Hardware locked",
+   "virtual device"},
+#endif
+#ifdef SANE_STATUS_WARMING_UP
+  {
+   "warmup",
+   "Noname",
+   "Always warming up",
+   "virtual device"},
+#endif
 };
 
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
   static const SANE_Device *devlist[] = {
-    dev + 0, dev + 1, 0
+    dev + 0, dev + 1,
+#ifdef SANE_STATUS_HW_LOCKED
+    dev + 2,
+#endif
+#ifdef SANE_STATUS_WARMING_UP
+    dev + 3,
+#endif
+    0
   };
 
   DBG (2, "sane_get_devices: local_only = %d\n", local_only);
@@ -576,6 +603,20 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       gamma[2][i] = i;
       gamma[3][i] = i;
     }
+
+#ifdef SANE_STATUS_HW_LOCKED
+  if(strncmp(devicename,"locked",6)==0)
+    return SANE_STATUS_HW_LOCKED;
+#endif
+
+#ifdef SANE_STATUS_WARMING_UP
+  if(strncmp(devicename,"warmup",6)==0)
+    {
+      warming_up = SANE_TRUE;
+      start.tv_sec = 0;
+    }
+#endif
+
   return SANE_STATUS_GOOD;
 }
 
@@ -1070,12 +1111,29 @@ sane_start (SANE_Handle handle)
 {
   char buf[1024];
   int nlines;
+#ifdef SANE_STATUS_WARMING_UP
+  struct timeval current;
+#endif
 
   DBG (2, "sane_start\n");
   rgb_comp = 0;
   if (handle != MAGIC || !is_open)
     return SANE_STATUS_INVAL;	/* Unknown handle ... */
 
+#ifdef SANE_STATUS_WARMING_UP
+  if(warming_up == SANE_TRUE)
+   {
+      gettimeofday(&current,NULL);
+      if(current.tv_sec-start.tv_sec>5)
+	{
+	   start.tv_sec = current.tv_sec;
+	   return SANE_STATUS_WARMING_UP;
+	}
+      if(current.tv_sec-start.tv_sec<5)
+	return SANE_STATUS_WARMING_UP;
+   }
+#endif
+
   if (infile != NULL)
     {
       fclose (infile);
