@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.35 2006/01/06 20:59:17 oliver-guest Exp $
+/* $Id$
    SANE SnapScan backend */
 
 /* default option values */
@@ -73,10 +73,10 @@ static SANE_Int def_bpp = 8;
 static char md_auto[] = "Auto";
 
 /* predefined scan mode names */
-static char md_colour[] = SANE_I18N("Color");
-static char md_bilevelcolour[] = SANE_I18N("Halftone");
-static char md_greyscale[] = SANE_I18N("Gray");
-static char md_lineart[] = SANE_I18N("Lineart");
+static char md_colour[] = SANE_VALUE_SCAN_MODE_COLOR;
+static char md_bilevelcolour[] = SANE_VALUE_SCAN_MODE_HALFTONE;
+static char md_greyscale[] = SANE_VALUE_SCAN_MODE_GRAY;
+static char md_lineart[] = SANE_VALUE_SCAN_MODE_LINEART;
 
 /* predefined scan source names */
 static char src_flatbed[] = SANE_I18N("Flatbed");
@@ -1616,7 +1616,7 @@ SANE_Status sane_control_option (SANE_Handle h,
 }
 
 /*
- * $Log: snapscan-options.c,v $
+ * $Log$
  * Revision 1.35  2006/01/06 20:59:17  oliver-guest
  * Some fixes for the Epson Stylus CX 1500
  *
