@@ -1,7 +1,7 @@
 #ifndef SP15C_SCSI_H
 #define SP15C_SCSI_H
 
-static const char RCSid_sh[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c-scsi.h,v 1.4 2003/12/27 17:48:38 hmg-guest Exp $";
+static const char RCSid_sh[] = "$Header$";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -48,7 +48,13 @@ static const char RCSid_sh[] = "$Header: /cvsroot/sane/sane-backends/backend/sp1
 
 /* ------------------------------------------------------------------------- */
 /*
- * $Log: sp15c-scsi.h,v $
+ * $Log$
+ * Revision 1.6  2005/09/19 19:57:48  fzago-guest
+ * Replaced __unused__ with __sane_unused__ to avoid a namespace conflict.
+ *
+ * Revision 1.5  2004/11/13 19:53:04  fzago-guest
+ * Fixes some warnings.
+ *
  * Revision 1.4  2003/12/27 17:48:38  hmg-guest
  * Silenced some compilation warnings.
  *
@@ -537,7 +543,7 @@ static scsiblk window_descriptor_blockB =
 
 /* ==================================================================== */
 
-static scsiblk *lint_catcher[] =
+static scsiblk __sane_unused__ *lint_catcher[] =
 {&reserve_unitB,
  &release_unitB,
  &inquiryB,
