@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
    
    This file is part of the SANE package.
    
@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: teco1.c,v 1.13 2004/02/08 15:27:55 fzago-guest Exp $
+   $Id$
    Some Relisys scanners AVEC and RELI series
 */
 
@@ -576,16 +576,13 @@ teco_get_inquiry_82 (Teco_Scanner * dev)
 /* SCSI sense handler. Callback for SANE.
  * These scanners never set asc or ascq. */
 static SANE_Status
-teco_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+teco_sense_handler (int __sane_unused__ scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int sensekey;
   int len;
 
   DBG (DBG_proc, "teco_sense_handler: enter\n");
 
-  arg = arg;			/* silence gcc */
-  scsi_fd = scsi_fd;		/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1471,7 +1468,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1481,15 +1478,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
-  DBG (DBG_error, "This is sane-teco1 version %d.%d-%d\n", V_MAJOR,
+  DBG (DBG_error, "This is sane-teco1 version %d.%d-%d\n", SANE_CURRENT_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
 
   if (version_code)
     {
-      *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+      *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     }
 
   fp = sanei_config_open (TECO_CONFIG_FILE);
@@ -1520,15 +1515,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2167,16 +2160,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (dev->scanning == SANE_FALSE)
     {
       return SANE_STATUS_INVAL;
@@ -2197,13 +2187,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ *fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
