@@ -1,7 +1,9 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+
+   Copyright (C) 2002-2006 Henning Meier-Geinitz <henning@meier-geinitz.de>
    Changes according to the sanei_thread usage by
                                          Gerhard Jaeger <gerhard@gjaeger.de>
+
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -43,7 +45,7 @@
    This backend is for testing frontends.
 */
 
-#define BUILD 27
+#define BUILD 28
 
 #include "../include/sane/config.h"
 
@@ -57,6 +59,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include "../include/_stdint.h"
+
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
@@ -119,7 +123,8 @@ static SANE_Range fixed_constraint_range = {
 };
 
 static SANE_String_Const mode_list[] = {
-  SANE_I18N ("Gray"), SANE_I18N ("Color"),
+  SANE_VALUE_SCAN_MODE_GRAY,
+  SANE_VALUE_SCAN_MODE_COLOR,
   0
 };
 
@@ -183,6 +188,11 @@ static SANE_Int int_array_constraint_word_list[] = {
   -42, 0, -8, 17, 42, 42
 };
 
+static SANE_String_Const source_list[] = {
+  SANE_I18N ("Flatbed"), SANE_I18N ("Automatic Document Feeder"),
+  0
+};
+
 static double random_factor;	/* use for fuzzyness of parameters */
 
 /* initial values */
@@ -192,11 +202,12 @@ static SANE_Fixed init_tl_y = SANE_FIX (0.0);
 static SANE_Fixed init_br_x = SANE_FIX (80.0);
 static SANE_Fixed init_br_y = SANE_FIX (100.0);
 static SANE_Word init_resolution = 50;
-static SANE_String init_mode = "Gray";
+static SANE_String init_mode =SANE_VALUE_SCAN_MODE_GRAY;
 static SANE_Word init_depth = 8;
 static SANE_Bool init_hand_scanner = SANE_FALSE;
 static SANE_Bool init_three_pass = SANE_FALSE;
 static SANE_String init_three_pass_order = "RGB";
+static SANE_String init_scan_source = "Flatbed";
 static SANE_String init_test_picture = "Solid black";
 static SANE_Bool init_invert_endianess = SANE_FALSE;
 static SANE_Bool init_read_limit = SANE_FALSE;
@@ -219,7 +230,7 @@ static SANE_Bool
 little_endian (void)
 {
   SANE_Int testvalue = 255;
-  u_int8_t *firstbyte = (u_int8_t *) & testvalue;
+  uint8_t *firstbyte = (uint8_t *) & testvalue;
 
   if (*firstbyte == 255)
     return SANE_TRUE;
@@ -287,6 +298,8 @@ init_options (Test_Device * test_device)
   od->constraint.range = 0;
   test_device->val[opt_num_opts].w = num_options;
 
+  test_device->loaded[opt_num_opts] = 1;
+
   /* opt_mode_group */
   od = &test_device->opt[opt_mode_group];
   od->name = "";
@@ -357,7 +370,7 @@ init_options (Test_Device * test_device)
   od->unit = SANE_UNIT_NONE;
   od->size = sizeof (SANE_Word);
   od->cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
-  if (strcmp (init_mode, "Color") != 0)
+  if (strcmp (init_mode, SANE_VALUE_SCAN_MODE_COLOR) != 0)
     od->cap |= SANE_CAP_INACTIVE;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
@@ -372,7 +385,7 @@ init_options (Test_Device * test_device)
   od->unit = SANE_UNIT_NONE;
   od->size = max_string_size (order_list);
   od->cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
-  if (strcmp (init_mode, "Color") != 0)
+  if (strcmp (init_mode, SANE_VALUE_SCAN_MODE_COLOR) != 0)
     od->cap |= SANE_CAP_INACTIVE;
   if (!init_three_pass)
     od->cap |= SANE_CAP_INACTIVE;
@@ -396,6 +409,22 @@ init_options (Test_Device * test_device)
   od->constraint.range = &resolution_range;
   test_device->val[opt_resolution].w = init_resolution;
 
+  /* opt_scan_source */
+  od = &test_device->opt[opt_scan_source];
+  od->name = SANE_NAME_SCAN_SOURCE;
+  od->title = SANE_TITLE_SCAN_SOURCE;
+  od->desc = SANE_I18N("If Automatic Document Feeder is selected, the feeder will be 'empty' after 10 scans.");
+  od->type = SANE_TYPE_STRING;
+  od->unit = SANE_UNIT_NONE;
+  od->size = max_string_size (source_list);
+  od->cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
+  od->constraint_type = SANE_CONSTRAINT_STRING_LIST;
+  od->constraint.string_list = source_list;
+  test_device->val[opt_scan_source].s = malloc (od->size);
+  if (!test_device->val[opt_scan_source].s)
+    return SANE_STATUS_NO_MEM;
+  strcpy (test_device->val[opt_scan_source].s, init_scan_source);
+
   /* opt_special_group */
   od = &test_device->opt[opt_special_group];
   od->name = "";
@@ -1324,16 +1353,16 @@ finish_pass (Test_Device * test_device)
       DBG (2, "finish_pass: pipe closed\n");
       test_device->pipe = -1;
     }
-  if (test_device->reader_pid > 0)
+  if (test_device->reader_pid != -1)
     {
       int status;
-      int pid;
+      SANE_Pid pid;
 
-      DBG (2, "finish_pass: terminating reader process %d\n",
-	   test_device->reader_pid);
+      DBG (2, "finish_pass: terminating reader process %ld\n",
+	   (long) test_device->reader_pid);
       sanei_thread_kill (test_device->reader_pid);
       pid = sanei_thread_waitpid (test_device->reader_pid, &status);
-      if (pid < 0)
+      if (pid == -1)
 	{
 	  DBG (1,
 	       "finish_pass: sanei_thread_waitpid failed, already terminated? (%s)\n",
@@ -1344,7 +1373,7 @@ finish_pass (Test_Device * test_device)
 	  DBG (2, "finish_pass: reader process terminated with status: %s\n",
 	       sane_strstatus (status));
 	}
-      test_device->reader_pid = 0;
+      test_device->reader_pid = -1;
     }
   /* this happens when running in thread context... */
   if (test_device->reader_fds >= 0)
@@ -1420,7 +1449,7 @@ print_options (Test_Device * test_device)
 
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * __sane_unused__ version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   SANE_Int linenumber;
@@ -1436,13 +1465,11 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   test_device = 0;
   previous_device = 0;
 
-  DBG (2, "sane_init: version_code= %p, authorize=%p\n",
-       (void *) version_code, (void *) authorize);
-  DBG (1, "sane_init: SANE test backend version %d.%d.%d from %s\n", V_MAJOR,
+  DBG (1, "sane_init: SANE test backend version %d.%d.%d from %s\n", SANE_CURRENT_MAJOR,
        V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
   if (inited)
     DBG (3, "sane_init: warning: already inited\n");
@@ -1508,6 +1535,9 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	  if (read_option (line, "depth", param_int,
 			   &init_depth) == SANE_STATUS_GOOD)
 	    continue;
+	  if (read_option (line, "scan-source", param_string,
+			   &init_scan_source) == SANE_STATUS_GOOD)
+	    continue;
 	  if (read_option (line, "test-picture", param_string,
 			   &init_test_picture) == SANE_STATUS_GOOD)
 	    continue;
@@ -1733,6 +1763,7 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   test_device->eof = SANE_FALSE;
   test_device->bytes_total = 0;
   test_device->pass = 0;
+  test_device->number_of_scans = 0;
 
   return SANE_STATUS_GOOD;
 }
@@ -1795,6 +1826,8 @@ sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
       return 0;
     }
 
+  test_device->loaded[option] = 1;
+
   return &test_device->opt[option];
 }
 
@@ -1835,6 +1868,12 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
       return SANE_STATUS_INVAL;
     }
 
+  if (!test_device->loaded[option])
+    {
+      DBG (1, "sane_control_option: option not loaded\n");
+      return SANE_STATUS_INVAL;
+    }
+
   if (!SANE_OPTION_IS_ACTIVE (test_device->opt[option].cap))
     {
       DBG (1, "sane_control_option: option is inactive\n");
@@ -2016,6 +2055,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
 	case opt_string:
 	case opt_string_constraint_string_list:
 	case opt_string_constraint_long_string_list:
+	case opt_scan_source:
 	  if (strcmp (test_device->val[option].s, value) == 0)
 	    {
 	      DBG (4, "sane_control_option: option %d (%s) not changed\n",
@@ -2055,7 +2095,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
 	  strcpy (test_device->val[option].s, (SANE_String) value);
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 	  myinfo |= SANE_INFO_RELOAD_OPTIONS;
-	  if (strcmp (test_device->val[option].s, "Color") == 0)
+	  if (strcmp (test_device->val[option].s, SANE_VALUE_SCAN_MODE_COLOR) == 0)
 	    {
 	      test_device->opt[opt_three_pass].cap &= ~SANE_CAP_INACTIVE;
 	      if (test_device->val[opt_three_pass].w == SANE_TRUE)
@@ -2237,6 +2277,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
 	case opt_string:
 	case opt_string_constraint_string_list:
 	case opt_string_constraint_long_string_list:
+	case opt_scan_source:
 	  strcpy (value, test_device->val[option].s);
 	  DBG (4, "sane_control_option: get option %d (%s), value=`%s'\n",
 	       option, test_device->opt[option].name, (SANE_String) value);
@@ -2269,12 +2310,22 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
       DBG (1, "sane_control_option: trying unexpected action %d\n", action);
       return SANE_STATUS_INVAL;
     }
+
   if (info)
     *info = myinfo;
+
+  if(myinfo & SANE_INFO_RELOAD_OPTIONS){
+    SANE_Int i = 0;
+    for(i=1;i<num_options;i++){
+      test_device->loaded[i] = 0;
+    }
+  }
+
   DBG (4, "sane_control_option: finished, info=%s %s %s \n",
        myinfo & SANE_INFO_INEXACT ? "inexact" : "",
        myinfo & SANE_INFO_RELOAD_PARAMS ? "reload_parameters" : "",
        myinfo & SANE_INFO_RELOAD_OPTIONS ? "reload_options" : "");
+
   return SANE_STATUS_GOOD;
 }
 
@@ -2339,7 +2390,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 	p->lines *= random_factor;
     }
 
-  if (strcmp (mode, "Gray") == 0)
+  if (strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY) == 0)
     {
       p->format = SANE_FRAME_GRAY;
       p->last_frame = SANE_TRUE;
@@ -2450,12 +2501,12 @@ sane_start (SANE_Handle handle)
     }
   if (test_device->scanning
       && (test_device->val[opt_three_pass].w == SANE_FALSE
-	  && strcmp (test_device->val[opt_mode].s, "Color") == 0))
+	  && strcmp (test_device->val[opt_mode].s, SANE_VALUE_SCAN_MODE_COLOR) == 0))
     {
       DBG (1, "sane_start: already scanning\n");
       return SANE_STATUS_INVAL;
     }
-  if (strcmp (test_device->val[opt_mode].s, "Color") == 0
+  if (strcmp (test_device->val[opt_mode].s, SANE_VALUE_SCAN_MODE_COLOR) == 0
       && test_device->val[opt_three_pass].w == SANE_TRUE
       && test_device->pass > 2)
     {
@@ -2463,6 +2514,19 @@ sane_start (SANE_Handle handle)
       return SANE_STATUS_INVAL;
     }
 
+  if (test_device->pass == 0)
+    {
+      test_device->number_of_scans++;
+      DBG (3, "sane_start: scanning page %d\n", test_device->number_of_scans);
+      
+      if ((strcmp (test_device->val[opt_scan_source].s, "Automatic Document Feeder") == 0) &&
+	  (((test_device->number_of_scans) % 11) == 0))
+	{
+	  DBG (1, "sane_start: Document feeder is out of documents!\n");
+	  return SANE_STATUS_NO_DOCS;
+	}
+    }
+
   test_device->scanning = SANE_TRUE;
   test_device->cancelled = SANE_FALSE;
   test_device->eof = SANE_FALSE;
@@ -2501,7 +2565,7 @@ sane_start (SANE_Handle handle)
   test_device->reader_pid =
     sanei_thread_begin (reader_task, (void *) test_device);
 
-  if (test_device->reader_pid < 0)
+  if (test_device->reader_pid == -1)
     {
       DBG (1, "sane_start: sanei_thread_begin failed (%s)\n",
 	   strerror (errno));
@@ -2627,7 +2691,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 	  return status;
 	}
       test_device->eof = SANE_TRUE;
-      if (strcmp (test_device->val[opt_mode].s, "Color") == 0
+      if (strcmp (test_device->val[opt_mode].s, SANE_VALUE_SCAN_MODE_COLOR) == 0
 	  && test_device->val[opt_three_pass].w == SANE_TRUE)
 	{
 	  test_device->pass++;
