@@ -2,7 +2,7 @@
    Uses the SANE library.
    Copyright (C) 1996, 1997, 1998 Andreas Beck and David Mosberger
    
-   Copyright (C) 1999 - 2005 by the SANE Project -- See AUTHORS and ChangeLog
+   Copyright (C) 1999 - 2009 by the SANE Project -- See AUTHORS and ChangeLog
    for details.
 
    For questions and comments contact the sane-devel mailinglist (see
@@ -23,14 +23,14 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifdef _AIX
-# include "../include/lalloca.h"	/* MUST come first for AIX! */
+# include "../include/lalloca.h"                /* MUST come first for AIX! */
 #endif
 
 #include "../include/sane/config.h"
 #include "../include/lalloca.h"
 
 #include <assert.h>
-#include <getopt.h>
+#include "lgetopt.h"
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,6 +41,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "../include/_stdint.h"
+
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
@@ -59,9 +61,8 @@
 
 typedef struct
 {
-  u_char *data;
-  int Bpp;			/* bytes/pixel */
-  int width;
+  uint8_t *data;
+  int width;    /*WARNING: this is in bytes, get pixel width from param*/
   int height;
   int x;
   int y;
@@ -86,8 +87,9 @@ static struct option basic_options[] = {
   {"verbose", no_argument, NULL, 'v'},
   {"progress", no_argument, NULL, 'p'},
   {"test", no_argument, NULL, 'T'},
+  {"all-options", no_argument, NULL, 'A'},
   {"version", no_argument, NULL, 'V'},
-  {"buffer_size", no_argument, NULL, 'B'},
+  {"buffer-size", optional_argument, NULL, 'B'},
   {"batch", optional_argument, NULL, 'b'},
   {"batch-count", required_argument, NULL, OPTION_BATCH_COUNT},
   {"batch-start", required_argument, NULL, OPTION_BATCH_START_AT},
@@ -104,7 +106,7 @@ static struct option basic_options[] = {
 #define OUTPUT_PNM      0
 #define OUTPUT_TIFF     1
 
-#define BASE_OPTSTRING	"d:hi:Lf:nvBVTbp"
+#define BASE_OPTSTRING	"d:hi:Lf:B::nvVTAbp"
 #define STRIP_HEIGHT	256	/* # lines we increment image height */
 
 static struct option *all_options;
@@ -114,15 +116,19 @@ static SANE_Handle device;
 static int verbose;
 static int progress = 0;
 static int test;
+static int all;
 static int output_format = OUTPUT_PNM;
 static int help;
 static int dont_scan = 0;
 static const char *prog_name;
-static SANE_Option_Descriptor window_option[2];
-static int window[4];
 static int resolution_optind = -1, resolution_value = 0;
-static SANE_Word window_val[2];
-static int window_val_user[2];	/* is width/height user-specified? */
+
+/* window (area) related options */
+static SANE_Option_Descriptor window_option[4]; /*updated descs for x,y,l,t*/
+static int window[4]; /*index into backend options for x,y,l,t*/
+static SANE_Word window_val[2]; /*the value for x,y options*/
+static int window_val_user[2];	/* is x,y user-specified? */
+
 static int accept_only_md5_auth = 0;
 static const char *icc_profile = NULL;
 
@@ -133,8 +139,6 @@ static SANE_Word tl_x = 0;
 static SANE_Word tl_y = 0;
 static SANE_Word br_x = 0;
 static SANE_Word br_y = 0;
-static SANE_Word w_x = 0;
-static SANE_Word h_y = 0;
 static SANE_Byte *buffer;
 static size_t buffer_size;
 
@@ -355,21 +359,51 @@ print_unit (SANE_Unit unit)
 }
 
 static void
-print_option (SANE_Device * device, int opt_num, char short_name)
+print_option (SANE_Device * device, int opt_num, const SANE_Option_Descriptor *opt)
 {
   const char *str, *last_break, *start;
-  const SANE_Option_Descriptor *opt;
   SANE_Bool not_first = SANE_FALSE;
   int i, column;
-  SANE_Word maxwindow = 0;
 
-  opt = sane_get_option_descriptor (device, opt_num);
+  if (opt->type == SANE_TYPE_GROUP){
+    printf ("  %s:\n", opt->title);
+    return;
+  }
 
-  if (short_name)
-    printf ("    -%c", short_name);
+  /* if both of these are set, option is invalid */
+  if(opt->cap & SANE_CAP_SOFT_SELECT && opt->cap & SANE_CAP_HARD_SELECT){
+    fprintf (stderr, "%s: invalid option caps, SS+HS\n", prog_name);
+    return;
+  }
+
+  /* invalid to select but not detect */
+  if(opt->cap & SANE_CAP_SOFT_SELECT && !(opt->cap & SANE_CAP_SOFT_DETECT)){
+    fprintf (stderr, "%s: invalid option caps, SS!SD\n", prog_name);
+    return;
+  }
+  /* standard allows this, though it makes little sense
+  if(opt->cap & SANE_CAP_HARD_SELECT && !(opt->cap & SANE_CAP_SOFT_DETECT)){
+    fprintf (stderr, "%s: invalid option caps, HS!SD\n", prog_name);
+    return;
+  }*/
+
+  /* if one of these three is not set, option is useless, skip it */
+  if(!(opt->cap & 
+   (SANE_CAP_SOFT_SELECT | SANE_CAP_HARD_SELECT | SANE_CAP_SOFT_DETECT)
+  )){
+    return;
+  }
+
+  /* print the option */
+  if ( !strcmp (opt->name, "x")
+    || !strcmp (opt->name, "y")
+    || !strcmp (opt->name, "t")
+    || !strcmp (opt->name, "l"))
+      printf ("    -%s", opt->name);
   else
     printf ("    --%s", opt->name);
 
+  /* print the option choices */
   if (opt->type == SANE_TYPE_BOOL)
     {
       fputs ("[=(", stdout);
@@ -402,26 +436,25 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 	    default:
 	      break;
 	    }
-	  if (opt->type != SANE_TYPE_STRING && opt->size
-	      > (SANE_Int) sizeof (SANE_Word))
+	  if (opt->type != SANE_TYPE_STRING
+           && opt->size > (SANE_Int) sizeof (SANE_Word))
 	    fputs (",...", stdout);
 	  break;
 
 	case SANE_CONSTRAINT_RANGE:
 	  if (opt->type == SANE_TYPE_INT)
 	    {
-	      if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
+	      if (!strcmp (opt->name, "x"))
 		{
-
-		  maxwindow = (opt->constraint.range->max) - tl_x;
-
-		  printf ("%d..%d", opt->constraint.range->min, maxwindow);
+		  printf ("%d..%d",
+                          opt->constraint.range->min, 
+                          opt->constraint.range->max - tl_x);
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_Y) == 0)
+	      else if (!strcmp (opt->name, "y"))
 		{
-		  maxwindow = (opt->constraint.range->max) - tl_y;
-
-		  printf ("%d..%d", opt->constraint.range->min, maxwindow);
+		  printf ("%d..%d",
+                          opt->constraint.range->min,
+                          opt->constraint.range->max - tl_y);
 		}
 	      else
 		{
@@ -437,22 +470,17 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 	    }
 	  else
 	    {
-	      if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
+	      if (!strcmp (opt->name, "x"))
 		{
-		  maxwindow = (opt->constraint.range->max) - tl_x;
-
 		  printf ("%g..%g",
 			  SANE_UNFIX (opt->constraint.range->min),
-			  SANE_UNFIX (maxwindow));
-
+			  SANE_UNFIX (opt->constraint.range->max - tl_x));
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_Y) == 0)
+	      else if (!strcmp (opt->name, "y"))
 		{
-		  maxwindow = (opt->constraint.range->max) - tl_y;
-
 		  printf ("%g..%g",
 			  SANE_UNFIX (opt->constraint.range->min),
-			  SANE_UNFIX (maxwindow));
+			  SANE_UNFIX (opt->constraint.range->max - tl_y));
 		}
 	      else
 		{
@@ -498,9 +526,10 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 	  break;
 	}
     }
+
+  /* print current option value */
   if (opt->type == SANE_TYPE_STRING || opt->size == sizeof (SANE_Word))
     {
-      /* print current option value */
       if (SANE_OPTION_IS_ACTIVE (opt->cap))
 	{
 	  void *val = alloca (opt->size);
@@ -514,27 +543,25 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 	      break;
 
 	    case SANE_TYPE_INT:
-	      if (strcmp (opt->name, SANE_NAME_SCAN_TL_X) == 0)
+	      if (strcmp (opt->name, "l") == 0)
 		{
 		  tl_x = (*(SANE_Fixed *) val);
 		  printf ("%d", tl_x);
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_TL_Y) == 0)
+	      else if (strcmp (opt->name, "t") == 0)
 		{
 		  tl_y = (*(SANE_Fixed *) val);
 		  printf ("%d", tl_y);
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
+	      else if (strcmp (opt->name, "x") == 0)
 		{
 		  br_x = (*(SANE_Fixed *) val);
-		  w_x = br_x - tl_x;
-		  printf ("%d", w_x);
+		  printf ("%d", br_x - tl_x);
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_Y) == 0)
+	      else if (strcmp (opt->name, "y") == 0)
 		{
 		  br_y = (*(SANE_Fixed *) val);
-		  h_y = br_y - tl_y;
-		  printf ("%d", h_y);
+		  printf ("%d", br_y - tl_y);
 		}
 	      else
 		printf ("%d", *(SANE_Int *) val);
@@ -542,27 +569,25 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 
 	    case SANE_TYPE_FIXED:
 
-	      if (strcmp (opt->name, SANE_NAME_SCAN_TL_X) == 0)
+	      if (strcmp (opt->name, "l") == 0)
 		{
 		  tl_x = (*(SANE_Fixed *) val);
 		  printf ("%g", SANE_UNFIX (tl_x));
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_TL_Y) == 0)
+	      else if (strcmp (opt->name, "t") == 0)
 		{
 		  tl_y = (*(SANE_Fixed *) val);
 		  printf ("%g", SANE_UNFIX (tl_y));
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
+	      else if (strcmp (opt->name, "x") == 0)
 		{
 		  br_x = (*(SANE_Fixed *) val);
-		  w_x = br_x - tl_x;
-		  printf ("%g", SANE_UNFIX (w_x));
+		  printf ("%g", SANE_UNFIX (br_x - tl_x));
 		}
-	      else if (strcmp (opt->name, SANE_NAME_SCAN_BR_Y) == 0)
+	      else if (strcmp (opt->name, "y") == 0)
 		{
 		  br_y = (*(SANE_Fixed *) val);
-		  h_y = br_y - tl_y;
-		  printf ("%g", SANE_UNFIX (h_y));
+		  printf ("%g", SANE_UNFIX (br_y - tl_y));
 		}
 	      else
 		printf ("%g", SANE_UNFIX (*(SANE_Fixed *) val));
@@ -583,39 +608,37 @@ print_option (SANE_Device * device, int opt_num, char short_name)
   if (!SANE_OPTION_IS_ACTIVE (opt->cap))
     fputs (" [inactive]", stdout);
 
+  else if(opt->cap & SANE_CAP_HARD_SELECT)
+    fputs (" [hardware]", stdout);
+
+  else if(!(opt->cap & SANE_CAP_SOFT_SELECT) && opt->cap & SANE_CAP_SOFT_DETECT)
+    fputs (" [read-only]", stdout);
+
   fputs ("\n        ", stdout);
 
-  switch (short_name)
+  column = 8;
+  last_break = 0;
+  start = opt->desc;
+  for (str = opt->desc; *str; ++str)
     {
-    case 'x':
-      fputs ("Width of scan-area.", stdout);
-      break;
-
-    case 'y':
-      fputs ("Height of scan-area.", stdout);
-      break;
-
-    default:
-      column = 8;
-      last_break = 0;
-      start = opt->desc;
-      for (str = opt->desc; *str; ++str)
-	{
-	  ++column;
-	  if (*str == ' ')
-	    last_break = str;
-	  if (column >= 79 && last_break)
-	    {
-	      while (start < last_break)
-		fputc (*start++, stdout);
-	      start = last_break + 1;	/* skip blank */
-	      fputs ("\n        ", stdout);
-	      column = 8 + (str - start);
-	    }
-	}
-      while (*start)
-	fputc (*start++, stdout);
+      ++column;
+      if (*str == ' ')
+        last_break = str;
+      else if (*str == '\n'){
+        column=80;
+        last_break = str;
+      }
+      if (column >= 79 && last_break)
+        {
+          while (start < last_break)
+            fputc (*start++, stdout);
+          start = last_break + 1;	/* skip blank */
+          fputs ("\n        ", stdout);
+          column = 8 + (str - start);
+        }
     }
+  while (*start)
+    fputc (*start++, stdout);
   fputc ('\n', stdout);
 }
 
@@ -715,7 +738,14 @@ parse_scalar (const SANE_Option_Descriptor * opt, const char *str,
 	str += sizeof ("us") - 1;
       break;
     }
-  *value = v + 0.5;
+
+  if(v < 0){
+    *value = v - 0.5;
+  }
+  else{
+    *value = v + 0.5;
+  }
+
   return str;
 }
 
@@ -831,22 +861,41 @@ fetch_options (SANE_Device * device)
   const SANE_Option_Descriptor *opt;
   SANE_Int num_dev_options;
   int i, option_count;
+  SANE_Status status;
 
-  /* and now build the full table of long options: */
+  opt = sane_get_option_descriptor (device, 0);
+  if (opt == NULL)
+    {
+      fprintf (stderr, "Could not get option descriptor for option 0\n");
+      exit (1);
+    }
 
-  sane_control_option (device, 0, SANE_ACTION_GET_VALUE, &num_dev_options, 0);
+  status = sane_control_option (device, 0, SANE_ACTION_GET_VALUE,
+                                &num_dev_options, 0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      fprintf (stderr, "Could not get value for option 0: %s\n",
+               sane_strstatus (status));
+      exit (1);
+    }
 
+  /* build the full table of long options */
   option_count = 0;
-  for (i = 0; i < num_dev_options; ++i)
+  for (i = 1; i < num_dev_options; ++i)
     {
       opt = sane_get_option_descriptor (device, i);
+      if (opt == NULL)
+	{
+	  fprintf (stderr, "Could not get option descriptor for option %d\n",i);
+	  exit (1);
+	}
 
       if (!SANE_OPTION_IS_SETTABLE (opt->cap))
 	continue;
 
       option_number[option_count] = i;
 
-      all_options[option_count].name = (char *) opt->name;
+      all_options[option_count].name = (const char *) opt->name;
       all_options[option_count].flag = 0;
       all_options[option_count].val = 0;
 
@@ -871,27 +920,15 @@ fetch_options (SANE_Device * device)
 	  && opt->size == sizeof (SANE_Int)
 	  && (opt->unit == SANE_UNIT_MM || opt->unit == SANE_UNIT_PIXEL))
 	{
-	  if (strcmp (opt->name, SANE_NAME_SCAN_TL_X) == 0)
-	    {
-	      window[2] = i;
-	      all_options[option_count].val = 'l';
-	    }
-	  else if (strcmp (opt->name, SANE_NAME_SCAN_TL_Y) == 0)
-	    {
-	      window[3] = i;
-	      all_options[option_count].val = 't';
-	    }
-	  else if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
+	  if (strcmp (opt->name, SANE_NAME_SCAN_BR_X) == 0)
 	    {
 	      window[0] = i;
 	      all_options[option_count].name = "width";
 	      all_options[option_count].val = 'x';
 	      window_option[0] = *opt;
 	      window_option[0].title = "Scan width";
-	      window_option[0].desc = "Width of scanning area.";
-	      if (!window_val_user[0])
-		sane_control_option (device, i, SANE_ACTION_GET_VALUE,
-				     &window_val[0], 0);
+	      window_option[0].desc = "Width of scan-area.";
+	      window_option[0].name = "x";
 	    }
 	  else if (strcmp (opt->name, SANE_NAME_SCAN_BR_Y) == 0)
 	    {
@@ -900,10 +937,22 @@ fetch_options (SANE_Device * device)
 	      all_options[option_count].val = 'y';
 	      window_option[1] = *opt;
 	      window_option[1].title = "Scan height";
-	      window_option[1].desc = "Height of scanning area.";
-	      if (!window_val_user[1])
-		sane_control_option (device, i, SANE_ACTION_GET_VALUE,
-				     &window_val[1], 0);
+	      window_option[1].desc = "Height of scan-area.";
+	      window_option[1].name = "y";
+	    }
+	  else if (strcmp (opt->name, SANE_NAME_SCAN_TL_X) == 0)
+	    {
+	      window[2] = i;
+	      all_options[option_count].val = 'l';
+	      window_option[2] = *opt;
+	      window_option[2].name = "l";
+	    }
+	  else if (strcmp (opt->name, SANE_NAME_SCAN_TL_Y) == 0)
+	    {
+	      window[3] = i;
+	      all_options[option_count].val = 't';
+	      window_option[3] = *opt;
+	      window_option[3].name = "t";
 	    }
 	}
       ++option_count;
@@ -916,12 +965,16 @@ fetch_options (SANE_Device * device)
      values for top-left x/y and bottom-right x/y: */
   for (i = 0; i < 2; ++i)
     {
-      if (window[i] && window[i + 2] && !window_val_user[i])
+      if (window[i] && !window_val_user[i])
 	{
-	  SANE_Word pos;
-	  sane_control_option (device, window[i + 2],
+	  sane_control_option (device, window[i],
+                                SANE_ACTION_GET_VALUE, &window_val[i], 0);
+          if (window[i + 2]){
+	    SANE_Word pos;
+	    sane_control_option (device, window[i + 2],
 			       SANE_ACTION_GET_VALUE, &pos, 0);
-	  window_val[i] = window_val[i] - pos + 1;
+	    window_val[i] -= pos;
+          }
 	}
     }
 }
@@ -932,7 +985,7 @@ set_option (SANE_Handle device, int optnum, void *valuep)
   const SANE_Option_Descriptor *opt;
   SANE_Status status;
   SANE_Word orig = 0;
-  SANE_Int info;
+  SANE_Int info = 0;
 
   opt = sane_get_option_descriptor (device, optnum);
   if (opt && (!SANE_OPTION_IS_ACTIVE (opt->cap)))
@@ -1106,10 +1159,10 @@ advance (Image * image)
 	  size_t old_size = 0, new_size;
 
 	  if (image->data)
-	    old_size = image->height * image->width * image->Bpp;
+	    old_size = image->height * image->width;
 
 	  image->height += STRIP_HEIGHT;
-	  new_size = image->height * image->width * image->Bpp;
+	  new_size = image->height * image->width;
 
 	  if (image->data)
 	    image->data = realloc (image->data, new_size);
@@ -1132,7 +1185,7 @@ scan_it (void)
   SANE_Byte min = 0xff, max = 0;
   SANE_Parameters parm;
   SANE_Status status;
-  Image image = { 0, 0, 0, 0, 0, 0 };
+  Image image = { 0, 0, 0, 0, 0 };
   static const char *format_name[] = {
     "gray", "RGB", "red", "green", "blue"
   };
@@ -1145,7 +1198,15 @@ scan_it (void)
     {
       if (!first_frame)
 	{
+#ifdef SANE_STATUS_WARMING_UP
+          do
+	    {
+	      status = sane_start (device);
+	    }
+	  while(status == SANE_STATUS_WARMING_UP);
+#else
 	  status = sane_start (device);
+#endif
 	  if (status != SANE_STATUS_GOOD)
 	    {
 	      fprintf (stderr, "%s: sane_start: %s\n",
@@ -1177,8 +1238,9 @@ scan_it (void)
 			 prog_name, parm.pixels_per_line,
 			 8 * parm.bytes_per_line / parm.pixels_per_line);
 	    }
+
 	  fprintf (stderr, "%s: acquiring %s frame\n", prog_name,
-		   format_name[parm.format]);
+	   parm.format <= SANE_FRAME_BLUE ? format_name[parm.format]:"Unknown");
 	}
 
       if (first_frame)
@@ -1215,16 +1277,20 @@ scan_it (void)
 				      parm.lines, parm.depth);
 		}
 	      break;
+
+            default:
+	      break;
 	    }
 
 	  if (must_buffer)
 	    {
 	      /* We're either scanning a multi-frame image or the
-	         scanner doesn't know what the eventual image height
-	         will be (common for hand-held scanners).  In either
-	         case, we need to buffer all data before we can write
-	         the image.  */
-	      image.width = parm.pixels_per_line;
+		 scanner doesn't know what the eventual image height
+		 will be (common for hand-held scanners).  In either
+		 case, we need to buffer all data before we can write
+		 the image.  */
+	      image.width = parm.bytes_per_line;
+
 	      if (parm.lines >= 0)
 		/* See advance(); we allocate one extra line so we
 		   don't end up realloc'ing in when the image has been
@@ -1232,11 +1298,7 @@ scan_it (void)
 		image.height = parm.lines - STRIP_HEIGHT + 1;
 	      else
 		image.height = 0;
-	      image.Bpp = 3;
-	      if (parm.format == SANE_FRAME_GRAY)
-		image.Bpp = 1;
-	      if (parm.depth == 16)
-		image.Bpp *= 2;
+
 	      image.x = image.width - 1;
 	      image.y = -1;
 	      if (!advance (&image))
@@ -1280,6 +1342,7 @@ scan_it (void)
 		}
 	      break;
 	    }
+
 	  if (must_buffer)
 	    {
 	      switch (parm.format)
@@ -1303,8 +1366,7 @@ scan_it (void)
 		  for (i = 0; i < len; ++i)
 		    {
 		      image.data[offset + i] = buffer[i];
-		      if (image.Bpp == 3 || (offset + i) % 2 == 0)
-			if ((offset + i) % 3 == 0 && !advance (&image))
+		      if (!advance (&image))
 			  {
 			    status = SANE_STATUS_NO_MEM;
 			    goto cleanup;
@@ -1317,8 +1379,7 @@ scan_it (void)
 		  for (i = 0; i < len; ++i)
 		    {
 		      image.data[offset + i] = buffer[i];
-		      if (image.Bpp == 1 || (offset + i) % 2 == 0)
-			if (!advance (&image))
+		      if (!advance (&image))
 			  {
 			    status = SANE_STATUS_NO_MEM;
 			    goto cleanup;
@@ -1326,6 +1387,9 @@ scan_it (void)
 		    }
 		  offset += len;
 		  break;
+
+                default:
+		  break;
 		}
 	    }
 	  else			/* ! must_buffer */
@@ -1383,29 +1447,32 @@ scan_it (void)
   if (must_buffer)
     {
       image.height = image.y;
+
       if (output_format == OUTPUT_TIFF)
 	sanei_write_tiff_header (parm.format, parm.pixels_per_line,
-				 parm.lines, parm.depth, resolution_value,
+				 image.height, parm.depth, resolution_value,
 				 icc_profile);
       else
-	write_pnm_header (parm.format, image.width, image.height, parm.depth);
-      if ((output_format == OUTPUT_TIFF) || (image.Bpp == 1)
-	  || (image.Bpp == 3))
-	fwrite (image.data, image.Bpp, image.height * image.width, stdout);
-      else			/* image.Bpp == 2 or image.Bpp == 6 assumed */
-	{
+	write_pnm_header (parm.format, parm.pixels_per_line,
+                          image.height, parm.depth);
+
 #if !defined(WORDS_BIGENDIAN)
+      /* multibyte pnm file may need byte swap to LE */
+      /* FIXME: other bit depths? */
+      if (output_format != OUTPUT_TIFF && parm.depth == 16)
+	{
 	  int i;
-	  for (i = 0; i < image.Bpp * image.height * image.width; i += 2)
+	  for (i = 0; i < image.height * image.width; i += 2)
 	    {
 	      unsigned char LSB;
 	      LSB = image.data[i];
 	      image.data[i] = image.data[i + 1];
 	      image.data[i + 1] = LSB;
 	    }
-#endif
-	  fwrite (image.data, image.Bpp, image.height * image.width, stdout);
 	}
+#endif
+
+	fwrite (image.data, 1, image.height * image.width, stdout);
     }
 
   /* flush the output buffer */
@@ -1442,7 +1509,7 @@ pass_fail (int max, int len, SANE_Byte * buffer, SANE_Status status)
     fprintf (stderr, "FAIL Error: %s\n", sane_strstatus (status));
   else if (buffer[len] != 0x23)
     {
-      while (buffer[len] != 0x23)
+      while (len <= max && buffer[len] != 0x23)
 	++len;
       fprintf (stderr, "FAIL Cheat: %d bytes\n", len);
     }
@@ -1460,11 +1527,20 @@ test_it (void)
   int i, len;
   SANE_Parameters parm;
   SANE_Status status;
-  Image image = { 0, 0, 0, 0, 0, 0 };
+  Image image = { 0, 0, 0, 0, 0 };
   static const char *format_name[] =
     { "gray", "RGB", "red", "green", "blue" };
 
+#ifdef SANE_STATUS_WARMING_UP
+  do
+    {
+      status = sane_start (device);
+    }
+  while(status == SANE_STATUS_WARMING_UP);
+#else
   status = sane_start (device);
+#endif
+
   if (status != SANE_STATUS_GOOD)
     {
       fprintf (stderr, "%s: sane_start: %s\n",
@@ -1489,8 +1565,9 @@ test_it (void)
 	     "variable height at %d bits/pixel\n",
 	     prog_name, parm.pixels_per_line,
 	     8 * parm.bytes_per_line / parm.pixels_per_line);
-  fprintf (stderr, "%s: acquiring %s frame, %d bits/sample\n",
-	   prog_name, format_name[parm.format], parm.depth);
+  fprintf (stderr, "%s: acquiring %s frame, %d bits/sample\n", prog_name,
+	   parm.format <= SANE_FRAME_BLUE ? format_name[parm.format]:"Unknown",
+           parm.depth);
 
   image.data = malloc (parm.bytes_per_line * 2);
 
@@ -1585,11 +1662,41 @@ scanimage_exit (void)
     fprintf (stderr, "scanimage: finished\n");
 }
 
+/** @brief print device options to stdout
+ *
+ * @param device struct of the opened device to describe
+ * @param num_dev_options number of device options
+ * @param ro SANE_TRUE to print read-only options
+ */
+static void print_options(SANE_Device * device, SANE_Int num_dev_options, SANE_Bool ro)
+{
+  int i, j;
+  const SANE_Option_Descriptor *opt;
+
+  for (i = 1; i < num_dev_options; ++i)
+    {
+      opt = 0;
+
+      /* scan area uses modified option struct */
+      for (j = 0; j < 4; ++j)
+	if (i == window[j])
+	  opt = window_option + j;
+
+      if (!opt)
+	opt = sane_get_option_descriptor (device, i);
+
+      if (ro || SANE_OPTION_IS_SETTABLE (opt->cap)
+	  || opt->type == SANE_TYPE_GROUP)
+	print_option (device, i, opt);
+    }
+  if (num_dev_options)
+    fputc ('\n', stdout);
+}
+
 int
 main (int argc, char **argv)
 {
   int ch, i, index, all_options_len;
-  const SANE_Option_Descriptor *opt;
   const SANE_Device **device_list;
   SANE_Int num_dev_options = 0;
   const char *devname = 0;
@@ -1652,11 +1759,17 @@ main (int argc, char **argv)
           progress = 1;
 	  break;
 	case 'B':
-	  buffer_size = (1024 * 1024);
+          if (optarg)
+	    buffer_size = 1024 * atoi(optarg);
+          else
+	    buffer_size = (1024 * 1024);
 	  break;
 	case 'T':
 	  test = 1;
 	  break;
+	case 'A':
+	  all = 1;
+	  break;
 	case 'n':
 	  dont_scan = 1;
 	  break;
@@ -1752,6 +1865,10 @@ main (int argc, char **argv)
 				int_arg = i;
 				ftype = 'i';
 				break;
+			      case 'n':
+				text_arg = "\n";
+				ftype = *percent = 's';
+				break;
 			      case '%':
 				ftype = 0;
 				break;
@@ -1793,10 +1910,10 @@ main (int argc, char **argv)
 	      }
 	    if (i == 0 && ch != 'f')
 	      printf ("\nNo scanners were identified. If you were expecting "
-		      "something different,\ncheck that the scanner is plugged "
-		      "in, turned on and detected by the\nsane-find-scanner tool "
-		      "(if appropriate). Please read the documentation\nwhich came "
-		      "with this software (README, FAQ, manpages).\n");
+                "something different,\ncheck that the scanner is plugged "
+		"in, turned on and detected by the\nsane-find-scanner tool "
+		"(if appropriate). Please read the documentation\nwhich came "
+		"with this software (README, FAQ, manpages).\n");
 
 	    if (defdevname)
 	      printf ("default device is `%s'\n", defdevname);
@@ -1819,9 +1936,11 @@ main (int argc, char **argv)
     {
       printf ("Usage: %s [OPTION]...\n\
 \n\
-Start image acquisition on a scanner device and write PNM image data to\n\
+Start image acquisition on a scanner device and write image data to\n\
 standard output.\n\
 \n\
+Parameters are separated by a blank from single-character options (e.g.\n\
+-d epson) and by a \"=\" from multi-character options (e.g. --device-name=epson).\n\
 -d, --device-name=DEVICE   use a given scanner device (e.g. hp:/dev/scanner)\n\
     --format=pnm|tiff      file format of output file\n\
 -i, --icc-profile=PROFILE  include this ICC profile into TIFF file\n", prog_name);
@@ -1829,24 +1948,26 @@ standard output.\n\
 -L, --list-devices         show available scanner devices\n\
 -f, --formatted-device-list=FORMAT similar to -L, but the FORMAT of the output\n\
                            can be specified: %%d (device name), %%v (vendor),\n\
-                           %%m (model), %%t (type), and %%i (index number)\n\
+                           %%m (model), %%t (type), %%i (index number), and\n\
+                           %%n (newline)\n\
 -b, --batch[=FORMAT]       working in batch mode, FORMAT is `out%%d.pnm' or\n\
                            `out%%d.tif' by default depending on --format\n");
       printf ("\
     --batch-start=#        page number to start naming files with\n\
     --batch-count=#        how many pages to scan in batch mode\n\
-    --batch-increment=#    increase number in filename by an amount of #\n\
-    --batch-double         increment page number by two for 2sided originals\n\
-                           being scanned in a single sided scanner\n\
+    --batch-increment=#    increase page number in filename by #\n\
+    --batch-double         increment page number by two, same as\n\
+                           --batch-increment=2\n\
     --batch-prompt         ask for pressing a key before scanning a page\n\
     --accept-md5-only      only accept authorization requests using md5\n");
       printf ("\
 -p, --progress             print progress messages\n\
 -n, --dont-scan            only set options, don't actually scan\n\
 -T, --test                 test backend thoroughly\n\
+-A, --all-options          list all available backend options\n\
 -h, --help                 display this help message and exit\n\
 -v, --verbose              give even more status messages\n\
--B, --buffer-size          change default input buffersize\n\
+-B, --buffer-size=#        change input buffer size (in kB, default 32)\n\
 -V, --version              print version information\n");
     }
 
@@ -1895,7 +2016,18 @@ standard output.\n\
 
   if (device)
     {
-      /* We got a device, find out how many options it has: */
+      const SANE_Option_Descriptor * desc_ptr;
+
+      /* Good form to always get the descriptor once before value */
+      desc_ptr = sane_get_option_descriptor(device, 0);
+      if (!desc_ptr)
+	{
+	  fprintf (stderr, "%s: unable to get option count descriptor\n",
+		   prog_name);
+	  exit (1);
+	}
+
+      /* We got a device, find out how many options it has */
       status = sane_control_option (device, 0, SANE_ACTION_GET_VALUE,
 				    &num_dev_options, 0);
       if (status != SANE_STATUS_GOOD)
@@ -1905,17 +2037,19 @@ standard output.\n\
 	  exit (1);
 	}
 
+      /* malloc global option lists */
       all_options_len = num_dev_options + NELEMS (basic_options) + 1;
       all_options = malloc (all_options_len * sizeof (all_options[0]));
       option_number_len = num_dev_options;
       option_number = malloc (option_number_len * sizeof (option_number[0]));
       if (!all_options || !option_number)
 	{
-	  fprintf (stderr, "%s: out of memory in fetch_options()\n",
+	  fprintf (stderr, "%s: out of memory in main()\n",
 		   prog_name);
 	  exit (1);
 	}
 
+      /* load global option lists */
       fetch_options (device);
 
       {
@@ -1953,8 +2087,10 @@ standard output.\n\
 	strcat (full_optstring, yarg);
       }
 
+      /* re-run argument processing with backend-specific options included
+       * this time, enable error printing and arg permutation */
       optind = 0;
-      opterr = 1;		/* re-enable error printing and arg permutation */
+      opterr = 1;
       while ((ch = getopt_long (argc, argv, full_optstring, all_options,
 				&index)) != EOF)
 	{
@@ -2006,54 +2142,40 @@ standard output.\n\
 	}
 
       free (full_optstring);
+
+      /* convert x/y to br_x/br_y */
       for (index = 0; index < 2; ++index)
 	if (window[index])
 	  {
-	    SANE_Word val, pos;
-	    pos = 0;
+            SANE_Word pos = 0;
+	    SANE_Word val = window_val[index];
 
-	    val = window_val[index] - 1;
 	    if (window[index + 2])
 	      {
 		sane_control_option (device, window[index + 2],
 				     SANE_ACTION_GET_VALUE, &pos, 0);
-		val = pos + window_val[index] - 1;
+		val += pos;
 	      }
 	    set_option (device, window[index], &val);
 	  }
+
+      /* output device-specific help */
       if (help)
 	{
 	  printf ("\nOptions specific to device `%s':\n", devname);
+	  print_options(device, num_dev_options, SANE_FALSE);
+	}
 
-	  for (i = 0; i < num_dev_options; ++i)
-	    {
-	      char short_name = '\0';
-	      int j;
-
-	      opt = 0;
-	      for (j = 0; j < 4; ++j)
-		if (i == window[j])
-		  {
-		    short_name = "xylt"[j];
-		    if (j < 2)
-		      opt = window_option + j;
-		  }
-	      if (!opt)
-		opt = sane_get_option_descriptor (device, i);
-
-	      if (opt->type == SANE_TYPE_GROUP)
-		printf ("  %s:\n", opt->title);
-
-	      if (!SANE_OPTION_IS_SETTABLE (opt->cap))
-		continue;
-
-	      print_option (device, i, short_name);
-	    }
-	  if (num_dev_options)
-	    fputc ('\n', stdout);
+      /*  list all device-specific options */
+      if (all)
+	{
+	  printf ("\nAll options specific to device `%s':\n", devname);
+	  print_options(device, num_dev_options, SANE_TRUE);
+          exit (0);
 	}
     }
 
+  /* output device list */
   if (help)
     {
       printf ("\
@@ -2113,11 +2235,21 @@ List of available devices:", prog_name);
 		 "Scanning %d pages, incrementing by %d, numbering from %d\n",
 		 batch_count, batch_increment, batch_start_at);
 
+      else if(isatty(fileno(stdout))){
+	fprintf (stderr,"%s: output is not a file, exiting\n", prog_name);
+        exit (1);
+      }
+
       do
 	{
 	  char path[PATH_MAX];
+	  char part_path[PATH_MAX];
 	  if (batch)		/* format is NULL unless batch mode */
-	    sprintf (path, format, n);	/* love --(C++) */
+	    {
+	      sprintf (path, format, n);	/* love --(C++) */
+	      strcpy (part_path, path);
+	      strcat (part_path, ".part");
+	    }
 
 
 	  if (batch)
@@ -2141,7 +2273,15 @@ List of available devices:", prog_name);
 	      fprintf (stderr, "Scanning page %d\n", n);
 	    }
 
+#ifdef SANE_STATUS_WARMING_UP
+          do
+	    {
+	      status = sane_start (device);
+	    }
+	  while(status == SANE_STATUS_WARMING_UP);
+#else
 	  status = sane_start (device);
+#endif
 	  if (status != SANE_STATUS_GOOD)
 	    {
 	      fprintf (stderr, "%s: sane_start: %s\n",
@@ -2150,9 +2290,10 @@ List of available devices:", prog_name);
 	      break;
 	    }
 
-	  if (batch && NULL == freopen (path, "w", stdout))
+	  /* write to .part file while scanning is in progress */
+	  if (batch && NULL == freopen (part_path, "w", stdout))
 	    {
-	      fprintf (stderr, "cannot open %s\n", path);
+	      fprintf (stderr, "cannot open %s\n", part_path);
 	      sane_cancel (device);
 	      return SANE_STATUS_ACCESS_DENIED;
 	    }
@@ -2167,15 +2308,36 @@ List of available devices:", prog_name);
 	  switch (status)
 	    {
 	    case SANE_STATUS_GOOD:
-	      break;
 	    case SANE_STATUS_EOF:
 	      status = SANE_STATUS_GOOD;
+	      if (batch)
+		{	
+		  /* close output file by redirecting, do not close
+		     stdout here! */
+		  if (NULL == freopen ("/dev/null", "w", stdout))
+		    {
+		      fprintf (stderr, "cannot open /dev/null\n");
+		      sane_cancel (device);
+		      return SANE_STATUS_ACCESS_DENIED;
+		    }
+		  else
+		    {
+		      /* let the fully scanned file show up */
+		      if (rename (part_path, path))
+			{
+			  fprintf (stderr, "cannot rename %s to %s\n",
+				part_path, path);
+			  sane_cancel (device);
+			  return SANE_STATUS_ACCESS_DENIED;
+			}
+		    }
+		}
 	      break;
 	    default:
 	      if (batch)
 		{
 		  fclose (stdout);
-		  unlink (path);
+		  unlink (part_path);
 		}
 	      break;
 	    }			/* switch */
