@@ -1,6 +1,8 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2003, 2005 Rene Rebe (sanei_read_int,sanei_set_timeout)
    Copyright (C) 2001, 2002 Henning Meier-Geinitz
+   Copyright (C) 2003, 2005 Rene Rebe (sanei_read_int,sanei_set_timeout)
+   Copyright (C) 2008 m. allan noah (sanei_usb_clear_halt)
+   Copyright (C) 2011 Reinhold Kainhofer (sanei_usb_set_endpoint)
    This file is part of the SANE package.
 
    SANE is free software; you can redistribute it and/or modify it
@@ -162,12 +164,40 @@
 #define USB_DIR_IN                      0x80
 /* @} */
 
+/** */
+struct sanei_usb_dev_descriptor
+{
+	SANE_Byte    desc_type;
+	unsigned int bcd_usb;
+	unsigned int bcd_dev;
+	SANE_Byte    dev_class;
+	SANE_Byte    dev_sub_class;
+	SANE_Byte    dev_protocol;
+	SANE_Byte    max_packet_size;
+};
+
 /** Initialize sanei_usb.
  *
  * Call this before any other sanei_usb function.
  */
 extern void sanei_usb_init (void);
 
+/** Get the vendor and product ids by device name.
+ *
+ * @param devname 
+ * @param vendor vendor id
+ * @param product product id
+ *
+ * @return
+ * - SANE_STATUS_GOOD - if the ids could be determined
+ * - SANE_STATUS_INVAL - if the device is not found
+ * - SANE_STATUS_UNSUPPORTED - if this method is not supported with the current
+ *   access method
+ */
+SANE_Status
+sanei_usb_get_vendor_product_byname (SANE_String_Const devname,
+				     SANE_Word * vendor, SANE_Word * product);
+
 /** Get the vendor and product ids.
  *
  * Currently, only libusb devices and scanners supported by the Linux USB 
@@ -178,8 +208,8 @@ extern void sanei_usb_init (void);
  * @param vendor vendor id
  * @param product product id
  *
- *@return 
- * - SANE_STATUS_GOOD - if the ids could be determinded
+ * @return 
+ * - SANE_STATUS_GOOD - if the ids could be determined
  * - SANE_STATUS_UNSUPPORTED - if the OS doesn't support detection of ids
  */
 extern SANE_Status
@@ -223,6 +253,33 @@ sanei_usb_find_devices (SANE_Int vendor, SANE_Int product,
  */
 extern SANE_Status sanei_usb_open (SANE_String_Const devname, SANE_Int * dn);
 
+/** Set the endpoint for the USB communication
+ *
+ * Allows to switch to a different endpoint for the USB communication than
+ * the default (auto-detected) endpoint. This function can only be called
+ * after sanei_usb_open.
+ *
+ * @param dn device number
+ * @param ep_type type of endpoint to set (bitwise or of USB_DIR_IN/OUT and
+ *                USB_ENDPOINT_TYPE_BULK/CONTROL/INTERRUPT/ISOCHRONOUS
+ * @param ep endpoint to use for the given type
+ *
+ */
+extern void sanei_usb_set_endpoint (SANE_Int dn, SANE_Int ep_type, SANE_Int ep);
+
+/** Retrieve the endpoint used for the USB communication
+ *
+ * Returns the endpoint used for the USB communication of the given type.
+ * This function can only be called after sanei_usb_open.
+ *
+ * @param dn device number
+ * @param ep_type type of endpoint to retrieve (bitwise or of USB_DIR_IN/OUT
+ *                and USB_ENDPOINT_TYPE_BULK/CONTROL/INTERRUPT/ISOCHRONOUS
+ * @return endpoint used for the given type
+ *
+ */
+extern SANE_Int sanei_usb_get_endpoint (SANE_Int dn, SANE_Int ep_type);
+
 /** Close a USB device.
  * 
  * @param dn device number
@@ -239,6 +296,22 @@ extern void sanei_usb_set_timeout (SANE_Int timeout);
  */
 #define HAVE_SANEI_USB_SET_TIMEOUT
 
+/** Clear halt condition on bulk endpoints
+ * 
+ * @param dn device number
+ */
+extern SANE_Status sanei_usb_clear_halt (SANE_Int dn);
+
+/** Check if sanei_usb_clear_halt() is available.
+ */
+#define HAVE_SANEI_USB_CLEAR_HALT
+
+/** Reset device
+ * 
+ * @param dn device number
+ */
+extern SANE_Status sanei_usb_reset (SANE_Int dn);
+
 /** Initiate a bulk transfer read.
  *
  * Read up to size bytes from the device to buffer. After the read, size
@@ -367,7 +440,7 @@ sanei_usb_set_configuration (SANE_Int dn, SANE_Int configuration);
  * Change claim interface
  *
  * @param dn device number
- * @param interface, interface nummber
+ * @param interface_number interface number
  *
  * @return 
  * - SANE_STATUS_GOOD - on succes
@@ -378,14 +451,14 @@ sanei_usb_set_configuration (SANE_Int dn, SANE_Int configuration);
  */
 
 extern SANE_Status
-sanei_usb_claim_interface (SANE_Int dn, SANE_Int interface);
+sanei_usb_claim_interface (SANE_Int dn, SANE_Int interface_number);
 
 /** Initiate release interface.
  *
  * Change release interface
  *
  * @param dn device number
- * @param interface, interface nummber
+ * @param interface_number interface number
  *
  * @return 
  * - SANE_STATUS_GOOD - on succes
@@ -396,7 +469,7 @@ sanei_usb_claim_interface (SANE_Int dn, SANE_Int interface);
  */
 
 extern SANE_Status
-sanei_usb_release_interface (SANE_Int dn, SANE_Int interface);
+sanei_usb_release_interface (SANE_Int dn, SANE_Int interface_number);
 
 /** Initiate a set altinterface.
  *
@@ -416,5 +489,24 @@ sanei_usb_release_interface (SANE_Int dn, SANE_Int interface);
 extern SANE_Status
 sanei_usb_set_altinterface (SANE_Int dn, SANE_Int alternate);
 
+/** Get some information from the device descriptor
+ *
+ * Sometimes it's useful to know something about revisions and
+ * other stuff reported by the USB system
+ *
+ * @param dn device number
+ * @param desc where to put the information to
+ *
+ * @return 
+ * - SANE_STATUS_GOOD - on succes
+ * - SANE_STATUS_UNSUPPORTED - if the feature is not supported by the OS or
+ *   SANE.
+ * - SANE_STATUS_INVAL - on every other error
+ *
+ */
+
+extern SANE_Status
+sanei_usb_get_descriptor( SANE_Int dn, struct sanei_usb_dev_descriptor *desc );
+
 /*------------------------------------------------------*/
 #endif /* sanei_usb_h */
