@@ -5,6 +5,8 @@
 *  2 of the License, or (at your option) any later version.
 */
 
+/* For putenv */
+#define _XOPEN_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -19,7 +21,7 @@ static void
 Usage (char *name)
 {
   fprintf (stderr,
-	   "%s [-c color_mode] [-x coord] [-y coord] [-w width] [-h height] [-g brightness] [-z contrast] [-d dpi] [-t level] [-s] [-p] [-l 0|1] [-a ioport_addr] [-r]\n",
+	   "%s [-c color_mode] [-x coord] [-y coord] [-w width] [-h height] [-g gain] [-z offset] [-d dpi] [-t level] [-s] [-p] [-l 0|1] [-a ioport_addr] [-r]\n",
 	   name);
 }
 
@@ -29,23 +31,26 @@ main (int argc, char **argv)
 {
   char dbgstr[80];
   int probe = 0;
-  int port = 0x378;
+  int port = 0;
   char *name = NULL;
   int scan = 0;
   int lamp = -1;
-  int i;
+  int i, fd;
   int found;
   int recover = 0;
   int trace = 0;
+  int maxw, maxh;
 
 /* scanning parameters : defaults to preview (75 dpi color, full scan area) */
-  int brightness = 0x0;
-  int contrast = 0x2C0;
+  int gain = 0x0;
+  int offset = 0x646;
   int dpi = 75;
   int x = 0, y = 0;
-  int width = 5100, height = 7000;
+  int width = -1, height = -1;
   int color = RGB_MODE;
 
+  char **ports;
+  int rc;
 
 
   /* option parsing */
@@ -61,8 +66,8 @@ main (int argc, char **argv)
      -t --trace   : execution trace
      -l --lamp    : turn lamp on/off 1/0
      -d --dpi     : set scan resolution
-     -g --brightness   : set RVB brightness
-     -z --contrast: set contrast
+     -g --gain   : set RVB gain
+     -z --offset: set offset
      -a --addr    : io port address
      -n --name    : ppdev device name
      -r           : recover from previous failed scan
@@ -89,7 +94,7 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected color mode value\n");
-	      return (0);
+	      return 0;
 	    }
 	  color = 0;
 	  i++;
@@ -106,7 +111,7 @@ main (int argc, char **argv)
 	    {
 	      fprintf (stderr, "unexpected color mode value <%s>\n", argv[i]);
 	      fprintf (stderr, "Must be RGB, RGB12, BW, or BW12\n");
-	      return (0);
+	      return 0;
 	    }
 	}
 
@@ -117,14 +122,9 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected x value\n");
-	      return (0);
+	      return 0;
 	    }
 	  x = atoi (argv[i + 1]);
-	  if (x < 0 || x > 5100)
-	    {
-	      fprintf (stderr, "x must be between 0 and 5099\n");
-	      return (0);
-	    }
 	  i++;
 	  found = 1;
 	}
@@ -135,14 +135,9 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected y value\n");
-	      return (0);
+	      return 0;
 	    }
 	  y = atoi (argv[i + 1]);
-	  if (y < 0 || y > 7000)
-	    {
-	      fprintf (stderr, "y must be between 0 and 7000\n");
-	      return (0);
-	    }
 	  i++;
 	  found = 1;
 	}
@@ -153,20 +148,9 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected width value\n");
-	      return (0);
+	      return 0;
 	    }
 	  width = atoi (argv[i + 1]);
-	  if ((width < 1) || (width > 5100))
-	    {
-	      fprintf (stderr, "width must be between 1 and 5100\n");
-	      return (0);
-	    }
-	  if (x + width > 5100)
-	    {
-	      fprintf (stderr,
-		       "Right side of scan area exceed physical limits (x+witdh>5100)\n");
-	      return (0);
-	    }
 	  i++;
 	  found = 1;
 	}
@@ -178,20 +162,9 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected height value\n");
-	      return (0);
+	      return 0;
 	    }
 	  height = atoi (argv[i + 1]);
-	  if ((height < 1) || (height > 7000))
-	    {
-	      fprintf (stderr, "height must be between 1 and 7000\n");
-	      return (0);
-	    }
-	  if (y + height > 7100)
-	    {
-	      fprintf (stderr,
-		       "Bottom side of scan area exceed physical limits (y+height>7100)\n");
-	      return (0);
-	    }
 	  i++;
 	  found = 1;
 	}
@@ -202,7 +175,7 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected trace value\n");
-	      return (0);
+	      return 0;
 	    }
 	  trace = atoi (argv[i + 1]);
 	  i++;
@@ -220,6 +193,8 @@ main (int argc, char **argv)
       if ((strcmp (argv[i], "-s") == 0) || (strcmp (argv[i], "--scan") == 0))
 	{
 	  scan = 1;
+	  /* we have to probe again if we scan */
+	  probe = 1;
 	  found = 1;
 	}
 
@@ -229,13 +204,13 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected dpi value\n");
-	      return (0);
+	      return 0;
 	    }
 	  dpi = atoi (argv[i + 1]);
 	  if ((dpi < 75) || (dpi > 1200))
 	    {
 	      fprintf (stderr, "dpi value has to be between 75 and 1200\n");
-	      return (0);
+	      return 0;
 	    }
 	  if ((dpi != 75)
 	      && (dpi != 150)
@@ -243,50 +218,51 @@ main (int argc, char **argv)
 	    {
 	      fprintf (stderr,
 		       "dpi value has to be 75, 150, 300, 600 or 1200\n");
-	      return (0);
+	      return 0;
 	    }
 	  i++;
 	  found = 1;
 	}
 
-      if ((strcmp (argv[i], "-g") == 0) || (strcmp (argv[i], "--brightness") == 0))
+      if ((strcmp (argv[i], "-g") == 0)
+	  || (strcmp (argv[i], "--gain") == 0))
 	{
 	  if (i == (argc - 1))
 	    {
 	      Usage (argv[0]);
-	      fprintf (stderr, "expected hex brightness value ( ex: A59 )\n");
-	      return (0);
+	      fprintf (stderr, "expected hex gain value ( ex: A59 )\n");
+	      return 0;
 	    }
 	  i++;
 	  found = 1;
 	  if (strlen (argv[i]) != 3)
 	    {
 	      Usage (argv[0]);
-	      fprintf (stderr, "expected hex brightness value ( ex: A59 )\n");
-	      return (0);
+	      fprintf (stderr, "expected hex gain value ( ex: A59 )\n");
+	      return 0;
 	    }
-	  brightness = strtol (argv[i], NULL, 16);
+	  gain = strtol (argv[i], NULL, 16);
 	  sanei_umax_pp_setauto (0);
 	}
 
       if ((strcmp (argv[i], "-z") == 0)
-	  || (strcmp (argv[i], "--contrast") == 0))
+	  || (strcmp (argv[i], "--offset") == 0))
 	{
 	  if (i == (argc - 1))
 	    {
 	      Usage (argv[0]);
-	      fprintf (stderr, "expected hex contrast value ( ex: A59 )\n");
-	      return (0);
+	      fprintf (stderr, "expected hex offset value ( ex: A59 )\n");
+	      return 0;
 	    }
 	  i++;
 	  found = 1;
 	  if (strlen (argv[i]) != 3)
 	    {
 	      Usage (argv[0]);
-	      fprintf (stderr, "expected hex contrast value ( ex: A59 )\n");
-	      return (0);
+	      fprintf (stderr, "expected hex offset value ( ex: A59 )\n");
+	      return 0;
 	    }
-	  contrast = strtol (argv[i], NULL, 16);
+	  offset = strtol (argv[i], NULL, 16);
 	}
 
       if ((strcmp (argv[i], "-n") == 0) || (strcmp (argv[i], "--name") == 0))
@@ -296,7 +272,7 @@ main (int argc, char **argv)
 	      Usage (argv[0]);
 	      fprintf (stderr,
 		       "expected device name ( ex: /dev/parport0 )\n");
-	      return (0);
+	      return 0;
 	    }
 	  i++;
 	  found = 1;
@@ -308,7 +284,7 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected hex io port value ( ex: 3BC )\n");
-	      return (0);
+	      return 0;
 	    }
 	  i++;
 	  found = 1;
@@ -316,7 +292,7 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected hex io port value ( ex: 378 )\n");
-	      return (0);
+	      return 0;
 	    }
 	  port = strtol (argv[i], NULL, 16);
 	}
@@ -328,7 +304,7 @@ main (int argc, char **argv)
 	    {
 	      Usage (argv[0]);
 	      fprintf (stderr, "expected lamp value\n");
-	      return (0);
+	      return 0;
 	    }
 	  lamp = atoi (argv[i + 1]);
 	  i++;
@@ -339,7 +315,7 @@ main (int argc, char **argv)
 	{
 	  Usage (argv[0]);
 	  fprintf (stderr, "unexpected argument <%s>\n", argv[i]);
-	  return (0);
+	  return 0;
 	}
 
       /* next arg */
@@ -354,20 +330,57 @@ main (int argc, char **argv)
       putenv (dbgstr);
     }
 
-
-  /* set x origin left to right */
-  x = 144 + (5100 - x) - width;
-
-  /*  enable I/O */
-  /* parport_claim */
-  if (sanei_umax_pp_InitPort (port, name) != 1)
+  /* no address or device given */
+  rc = 0;
+  if ((name == NULL) && (port == 0))
     {
-      fprintf (stderr, "failed to gain direct acces to port 0x%X!\n", port);
-      return (0);
+      /* safe tests: user parallel port devices */
+      ports = sanei_parport_find_device ();
+      if (ports != NULL)
+	{
+	  i = 0;
+	  rc = 0;
+	  while ((ports[i] != NULL) && (rc != 1))
+	    {
+	      rc = sanei_umax_pp_initPort (port, ports[i]);
+	      i++;
+	    }
+	}
+
+      /* try for direct hardware access */
+      if (rc != 1)
+	{
+	  ports = sanei_parport_find_port ();
+	  i = 0;
+	  rc = 0;
+	  while ((ports[i] != NULL) && (rc != 1))
+	    {
+	      rc = sanei_umax_pp_initPort (strtol (ports[i], NULL, 16), NULL);
+	      i++;
+	    }
+	}
+      if (rc != 1)
+	{
+	  fprintf (stderr, "failed to detect a valid device or port!\n");
+	  return 0;
+	}
+    }
+  else
+    {
+      if (sanei_umax_pp_initPort (port, name) != 1)
+	{
+	  if (port)
+	    fprintf (stderr, "failed to gain direct acces to port 0x%X!\n",
+		     port);
+	  else
+	    fprintf (stderr, "failed to gain acces to device %s!\n", name);
+	  return 0;
+	}
     }
   if (trace)
     {
-      printf ("UMAX 1220P scanning program version 4.1 starting ...\n");
+      printf
+	("UMAX 610P/1220P/2000P scanning program version 6.4 starting ...\n");
 #ifdef HAVE_LINUX_PPDEV_H
       printf ("ppdev character device built-in.\n");
 #endif
@@ -381,70 +394,134 @@ main (int argc, char **argv)
   if ((!scan) && (lamp < 0) && (!probe))
     scan = 1;
 
-  /* lamp on/off */
-  if (lamp >= 0)
-    {
-      /* init transport layer */
-      if (sanei_umax_pp_InitTransport (recover) != 1)
-	{
-	  printf ("InitTransport() failed (%s:%d)\n", __FILE__, __LINE__);
-	  return (0);
-	}
-      if (sanei_umax_pp_SetLamp (lamp) == 0)
-	{
-	  fprintf (stderr, "Setting lamp state failed!\n");
-	  return (0);
-	}
-    }
-
 
   /* probe scanner */
-  if (probe)
+  if ((probe) || (lamp >= 0))
     {
       printf ("Probing scanner ....\n");
-      if (sanei_umax_pp_ProbeScanner (recover) != 1)
+      if (sanei_umax_pp_probeScanner (recover) != 1)
 	{
 	  if (recover)
 	    {
-	      sanei_umax_pp_InitTransport (recover);
-	      sanei_umax_pp_EndSession ();
-	      if (sanei_umax_pp_ProbeScanner (recover) != 1)
+	      sanei_umax_pp_initTransport (recover);
+	      sanei_umax_pp_endSession ();
+	      if (sanei_umax_pp_probeScanner (recover) != 1)
 		{
 		  printf ("Recover failed ....\n");
-		  return (0);
+		  return 0;
 		}
 	      printf ("Recover done !\n");
 	    }
 	  else
-	    return (0);
+	    return 0;
 	}
 
       /* could be written better .... but it is only test */
-      sanei_umax_pp_EndSession ();
+      sanei_umax_pp_endSession ();
+
       /* init transport layer */
-      if (sanei_umax_pp_InitTransport (0) != 1)
+      if (sanei_umax_pp_initTransport (0) != 1)
 	{
-	  printf ("InitTransport() failed (%s:%d)\n", __FILE__, __LINE__);
-	  return (0);
+	  printf ("initTransport() failed (%s:%d)\n", __FILE__, __LINE__);
+	  return 0;
 	}
-      i = sanei_umax_pp_CheckModel ();
-      if (i < 610)
+      i = sanei_umax_pp_checkModel ();
+      if (i < 600)
 	{
-	  sanei_umax_pp_EndSession ();
-	  printf ("CheckModel() failed (%s:%d)\n", __FILE__, __LINE__);
-	  return (0);
+	  sanei_umax_pp_endSession ();
+	  printf ("checkModel() failed (%s:%d)\n", __FILE__, __LINE__);
+	  return 0;
 	}
-      if (trace)
-	printf ("UMAX Astra %dP detected \n", i);
+      printf ("UMAX Astra %dP detected \n", i);
 
       /* free scanner if a scan is planned */
       if (scan)
-	sanei_umax_pp_EndSession ();
+	sanei_umax_pp_endSession ();
+      printf ("Done ....\n");
+    }
+
+  /* lamp on/off: must come after probing (610p handling) */
+  if (lamp >= 0)
+    {
+      /* init transport layer */
+      if (trace)
+	printf ("Tryning to set lamp %s\n", lamp ? "on" : "off");
+      if (sanei_umax_pp_initTransport (recover) != 1)
+	{
+	  printf ("initTransport() failed (%s:%d)\n", __FILE__, __LINE__);
+	  return 0;
+	}
+      else
+	{
+	  if (trace)
+	    printf ("initTransport passed...\n");
+	}
+      if (sanei_umax_pp_setLamp (lamp) == 0)
+	{
+	  fprintf (stderr, "Setting lamp state failed!\n");
+	  return 0;
+	}
+      else
+	{
+	  if (trace)
+	    printf ("sanei_umax_pp_setLamp passed...\n");
+	}
     }
 
   /* scan */
   if (scan)
     {
+      printf ("Scanning ....\n");
+      if (sanei_umax_pp_getastra () < 1210)
+	{
+	  maxw = 2550;
+	  maxh = 3500;
+	}
+      else
+	{
+	  maxw = 5100;
+	  maxh = 7000;
+	}
+      if (width < 0)
+	width = maxw;
+      if (height < 0)
+	height = maxh;
+
+      if ((width < 1) || (width > maxw))
+	{
+	  fprintf (stderr, "width must be between 1 and %d\n", maxw);
+	  return 0;
+	}
+      if (x + width > maxw)
+	{
+	  fprintf (stderr,
+		   "Right side of scan area exceed physical limits (x+witdh>%d)\n",
+		   maxw);
+	  return 0;
+	}
+      if (y < 0 || y > maxh)
+	{
+	  fprintf (stderr, "y must be between 0 and %d\n", maxh - 1);
+	  return 0;
+	}
+      if (x < 0 || x > maxw)
+	{
+	  fprintf (stderr, "x must be between 0 and %d\n", maxw - 1);
+	  return 0;
+	}
+      if ((height < 1) || (height > maxh))
+	{
+	  fprintf (stderr, "height must be between 1 and %d\n", maxh);
+	  return 0;
+	}
+      if (y + height > maxh)
+	{
+	  fprintf (stderr,
+		   "Bottom side of scan area exceed physical limits (y+height>%d)\n",
+		   maxh);
+	  return 0;
+	}
+
       /* init transport layer */
       /* 0: failed
          1: success
@@ -452,43 +529,45 @@ main (int argc, char **argv)
        */
       do
 	{
-	  i = sanei_umax_pp_InitTransport (recover);
+	  i = sanei_umax_pp_initTransport (recover);
 	}
       while (i == 2);
       if (i != 1)
 	{
-	  printf ("InitTransport() failed (%s:%d)\n", __FILE__, __LINE__);
-	  return (0);
+	  printf ("initTransport() failed (%s:%d)\n", __FILE__, __LINE__);
+	  return 0;
 	}
-      /*i = sanei_umax_pp_CheckModel ();
-         if (i < 610)
-         {
-         sanei_umax_pp_EndSession ();
-         printf ("CheckModel() failed (%s:%d)\n", __FILE__, __LINE__);
-         return (0);
-         }
-         if (trace)
-         printf ("UMAX Astra %dP detected \n", i); */
       /* init scanner */
-      if (sanei_umax_pp_InitScanner (recover) == 0)
+      if (sanei_umax_pp_initScanner (recover) == 0)
 	{
-	  sanei_umax_pp_EndSession ();
-	  return (0);
+	  sanei_umax_pp_endSession ();
+	  return 0;
 	}
+
+      /* set x origin left to right */
+      x = sanei_umax_pp_getLeft () + (maxw - x) - width;
+
       /* scan */
-      if (sanei_umax_pp_Scan
-	  (x, y, width, height, dpi, color, brightness, contrast) != 1)
+      if (sanei_umax_pp_scan
+	  (x, y, width, height, dpi, color, gain, offset) != 1)
 	{
-	  sanei_umax_pp_ReleaseScanner ();
-	  sanei_umax_pp_EndSession ();
-	  return (0);
+	  sanei_umax_pp_endSession ();
+	  return 0;
 	}
 
       /* wait for head parking */
-      sanei_umax_pp_ParkWait ();
-      sanei_umax_pp_ReleaseScanner ();
+      sanei_umax_pp_parkWait ();
+      printf ("Done ....\n");
     }
-  sanei_umax_pp_EndSession ();
+  sanei_umax_pp_endSession ();
+#ifdef HAVE_LINUX_PPDEV_H
+  fd = sanei_umax_pp_getparport ();
+  if (fd > 0)
+    {
+      close (fd);
+      sanei_umax_pp_setparport (0);
+    }
+#endif
 
-  return (1);
+  return 1;
 }
